// GUIBASE.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "guibase.h"

#ifdef ENABLE_GUI
#include "window.h"

#include <strstream>

////////////////////////////////////////////////////////////////////////////////////////////////////

iGLu bs_gui::list_counter = 1;		// zero is not a valid display list number...

bs_gui::bs_gui(window_factory & p1, ostream * p2) : bs_mdl_simple(p2)
{
	factory = & p1;
	
	select_buffer = new iGLu[SB_SIZE];
	selected_object = NULL;
}

bs_gui::~bs_gui(void)
{
	for (i32u n1 = 0;n1 < light_vector.size();n1++) delete light_vector[n1];
	for (i32u n1 = 0;n1 < object_vector.size();n1++) delete object_vector[n1];
	delete[] select_buffer;
}

iGLu bs_gui::GetDisplayListIDs(iGLu p1)
{
	iGLu first = list_counter;
	list_counter += p1;
	
	return first;
}

void bs_gui::DeleteDisplayLists(iGLu p1, iGLu p2)
{
	for (i32u n1 = 0;n1 < view_window_vector.size();n1++)
	{
		view_window_vector[n1]->SetCurrent();
		glDeleteLists(p1, p2);
	}
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

i32s bs_gui::IsLight(const dummy_object * p1)
{
	i32s index = NOT_DEFINED;
	for (i32u n1 = 0;n1 < light_vector.size();n1++)
	{
		if (light_vector[n1] == p1) index = n1;
	}
	
	return index;
}

bool bs_gui::SelectLight(const dummy_object * p1)
{
	i32s n1 = IsLight(p1);
	if (n1 < 0) return false;
	
	selected_object = light_vector[n1];
	
	cout << "selected " << light_vector[n1]->GetObjectName() << " which is ";
	cout << (light_vector[n1]->owner != NULL ? "local " : "global ") << endl;
	
	return true;
}

bool bs_gui::AddGlobalLight(light * p1)
{
	iGLs max_local_size = 0;
	for (i32u n1 = 0;n1 < camera_vector.size();n1++)
	{
		iGLs local_size = CountLocalLights(camera_vector[n1]);
		if (local_size > max_local_size) max_local_size = local_size;
	}
	
	iGLs total_lights = CountGlobalLights() + max_local_size;
	iGLs max_lights; glGetIntegerv(GL_MAX_LIGHTS, & max_lights);
	if (total_lights == max_lights) return false;
	
	light_vector.push_back(p1);
	SetGlobalLightNumbers();
	
	for (i32u n1 = 0;n1 < camera_vector.size();n1++)
	{
		SetLocalLightNumbers(camera_vector[n1]);
		SetupLights(camera_vector[n1]);
	}
	
	selected_object = light_vector.back();
	return true;
}

bool bs_gui::AddLocalLight(light * p1, camera * p2)
{
	iGLs total_lights = CountGlobalLights() + CountLocalLights(p2);
	iGLs max_lights; glGetIntegerv(GL_MAX_LIGHTS, & max_lights);
	if (total_lights == max_lights) return false;
	
	p1->owner = p2; light_vector.push_back(p1);
	SetLocalLightNumbers(p2); SetupLights(p2);
	
	selected_object = light_vector.back();
	return true;
}

bool bs_gui::RemoveLight(dummy_object * p1)
{
	i32s n1 = IsLight(p1); if (n1 < 0) return false;
	
	camera * owner = light_vector[n1]->owner;
	light_vector.erase(light_vector.begin() + n1);
	delete p1;
	
	if (owner != NULL)
	{
		SetLocalLightNumbers(owner);
		SetupLights(owner);
	}
	else
	{
		SetGlobalLightNumbers();
		for (i32u n1 = 0;n1 < camera_vector.size();n1++)
		{
			SetLocalLightNumbers(camera_vector[n1]);
			SetupLights(camera_vector[n1]);
		}
	}
	
	return true;
}

iGLs bs_gui::CountGlobalLights(void)
{
	iGLs sum = 0; i32u n1 = 0;
	while (n1 < light_vector.size())
	{
		if (light_vector[n1++]->owner == NULL) sum++;
	}
	
	return sum;
}

iGLs bs_gui::CountLocalLights(camera * p1)
{
	iGLs sum = 0; i32u n1 = 0;
	while (n1 < light_vector.size())
	{
		if (light_vector[n1++]->owner == p1) sum++;
	}
	
	return sum;
}

bool bs_gui::SetupLights(camera * p1)
{
	if (p1->gui != this) return false;
	iGLs max_lights; glGetIntegerv(GL_MAX_LIGHTS, & max_lights);
	
	for (i32u n1 = 0;n1 < view_window_vector.size();n1++)
	{
		if (view_window_vector[n1]->cam != p1) continue;
		
		view_window_vector[n1]->SetCurrent();
		for (iGLs n2 = 0;n2 < max_lights;n2++)
		{
			glDisable((GLenum) (GL_LIGHT0 + n2));
		}
		
		for (i32u n2 = 0;n2 < light_vector.size();n2++)
		{
			bool test1 = (light_vector[n2]->owner != NULL);
			bool test2 = (light_vector[n2]->owner != p1);
			if (test1 && test2) continue;
			
			light_vector[n2]->SetupProperties(); bool test = false;
			if (light_vector[n2]->owner == NULL && p1->use_global_lights) test = true;
			if (light_vector[n2]->owner == p1 && p1->use_local_lights) test = true;
			if (test) glEnable((GLenum) light_vector[n2]->number);
		}
	}
	
	return true;
}

void bs_gui::SetGlobalLightNumbers(void)
{
	iGLs counter = 0;
	for (i32u n1 = 0;n1 < light_vector.size();n1++)
	{
		if (light_vector[n1]->owner != NULL) continue;
		light_vector[n1]->number = GL_LIGHT0 + counter++;
	}
}

void bs_gui::SetLocalLightNumbers(camera * p1)
{
	iGLs counter = CountGlobalLights();
	for (i32u n1 = 0;n1 < light_vector.size();n1++)
	{
		if (light_vector[n1]->owner != p1) continue;
		light_vector[n1]->number = GL_LIGHT0 + counter++;
	}
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

i32s bs_gui::IsObject(const dummy_object * p1)
{
	i32s index = NOT_DEFINED;
	for (i32u n1 = 0;n1 < object_vector.size();n1++)
	{
		if (object_vector[n1] == p1) index = n1;
	}
	
	return index;
}

bool bs_gui::SelectObject(const dummy_object * p1)
{
	i32s n1 = IsObject(p1);
	if (n1 < 0) return false;
	
	selected_object = object_vector[n1];
	
	cout << "selected object #" << n1 << " which is ";
	cout << object_vector[n1]->GetObjectName() << endl;
	
	return true;
}

void bs_gui::AddObject(smart_object * p1)
{
	object_vector.push_back(p1);
	selected_object = object_vector.back();
}

bool bs_gui::RemoveObject(dummy_object * p1)
{
	i32s n1 = IsObject(p1); if (n1 < 0) return false;
	object_vector.erase(object_vector.begin() + n1);
	delete p1; return true;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

view_window * bs_gui::AddCamera(fGL p1)
{
	camera_vector.push_back(new camera(ol_static(), p1, this));
	view_window_vector.push_back(factory->ProduceView(this, camera_vector.back()));
	AddLocalLight(new directional_light(ol_static()), camera_vector.back());
	UpdateAllWindowTitles(); return view_window_vector.back();
}

view_window * bs_gui::AddViewWindow(camera * p1)
{
	view_window_vector.push_back(factory->ProduceView(this, p1)); SetupLights(p1);
	UpdateAllWindowTitles(); return view_window_vector.back();
}

bool bs_gui::RemoveViewWindow(view_window * p1)
{
	i32s n1 = NOT_DEFINED;
	for (i32u n2 = 0;n2 < camera_vector.size();n2++)
	{
		if (camera_vector[n2] == p1->cam) n1 = n2;
	}
	
	if (n1 == NOT_DEFINED) return false;
	
	i32s n2 = NOT_DEFINED; i32s n3 = 0;
	for (i32u n4 = 0;n4 < view_window_vector.size();n4++)
	{
		if (view_window_vector[n4] == p1) n2 = n4;
		else if (view_window_vector[n4]->cam == p1->cam) n3++;
	}
	
	if (n2 == NOT_DEFINED) return false;
	
// is this test only TARGET2-related??????? it won't let close the last window?!?!?!?!?!?!?!
// is this test only TARGET2-related??????? it won't let close the last window?!?!?!?!?!?!?!
// is this test only TARGET2-related??????? it won't let close the last window?!?!?!?!?!?!?!
	
	if (!n3 && camera_vector.size() < 2) return false;
	
	delete view_window_vector[n2];
	view_window_vector.erase(view_window_vector.begin() + n2);
	
	if (!n3)
	{
		delete camera_vector[n1];
		camera_vector.erase(camera_vector.begin() + n1);
	}
	
	UpdateAllWindowTitles();
	return true;
}

void bs_gui::UpdateAllWindowTitles(void)
{
        char buffer[1024];
        
	for (i32u n1 = 0;n1 < camera_vector.size();n1++)
	{
		i32s count = 1;
		for (i32u n2 = 0;n2 < view_window_vector.size();n2++)
		{
			if (view_window_vector[n2]->cam != camera_vector[n1]) continue;
			
			ostrstream str(buffer, sizeof(buffer));
			
#ifdef TARGET3

			// in TARGET3, the project file names are also included...
			
			str << project_file_name << "." << GetProjectFileNameExtension() << " : ";
			
#endif	// TARGET3

			str << "camera #" << (n1 + 1) << " view #" << count++ << ends;
			view_window_vector[n2]->SetTitle(buffer);
		}
	}
}

void bs_gui::UpdateAllViewWindows(bool flag)
{
	for (i32u n1 = 0;n1 < view_window_vector.size();n1++)
	{
		view_window_vector[n1]->Update(flag);
	}
}

void bs_gui::UpdateViewWindows(camera * cam, bool flag)
{
	for (i32u n1 = 0;n1 < view_window_vector.size();n1++)
	{
		if (view_window_vector[n1]->cam != cam) continue;
		view_window_vector[n1]->Update(flag);
	}
}

void bs_gui::UpdateViewWindow(view_window * win, bool flag)
{
	win->Update(flag);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

void bs_gui::DoDeleteCurrentObject(void)
{
	if (selected_object != NULL)
	{
		bool test1 = RemoveLight(selected_object);
		bool test2 = test1; if (!test1) test2 = RemoveObject(selected_object);
		
		if (test2)
		{
			selected_object = NULL;
			UpdateAllViewWindows();
		}
	}
}

void bs_gui::DoSwitchLocalLights(camera * cam, bool report)
{
	cam->use_local_lights = !cam->use_local_lights;
	if (report) cout << "local lights = " << (cam->use_local_lights ? "on" : "off") << endl;
	SetupLights(cam); UpdateViewWindows(cam);
}

void bs_gui::DoSwitchGlobalLights(camera * cam, bool report)
{
	cam->use_global_lights = !cam->use_global_lights;
	if (report) cout << "global lights = " << (cam->use_global_lights ? "on" : "off") << endl;
	SetupLights(cam); UpdateViewWindows(cam);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI

// eof
