// MM1ALG.H : algorithms for mm1-models.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef MM1ALG_H
#define MM1ALG_H

class mm1_geomopt;
struct mm1_geomopt_param;

class mm1_moldyn;
struct mm1_moldyn_param;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "mm1eng.h"
#include "conjgrad.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

class mm1_geomopt : public conjugate_gradient
{
	public:
	
	mm1_eng * eng;
	
	public:
	
	mm1_geomopt(mm1_eng *, i32s, f64);
	~mm1_geomopt(void);
	
	f64 GetValue(void);		// virtual
	f64 GetGradient(void);		// virtual
};

struct mm1_geomopt_param
{
	i32s nsteps;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class mm1_moldyn
{
	public:
	
	mm1_eng * eng;
	
	f64 * vel;
	f64 * acc;
	
	f64 * mass;
	
	f64 temp;
	
	f64 step1; f64 step2;
	f64 kin; f64 pot;
	
	public:
	
	mm1_moldyn(mm1_eng *, f64, f64);
	~mm1_moldyn(void);
	
	void TakeMDStep(void);
	
	f64 KineticEnergy(void);
	
	f64 ConvTempEKin(f64);
	f64 ConvEKinTemp(f64);
	
	void SetEKin(f64);
};

struct mm1_moldyn_param
{
	i32s nsteps;
	f64 timestep;
	
	char filename[256];
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// MM1ALG_H

// eof
