// MM1MDL.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "mm1mdl.h"
#include "v3d.h"

#include <fstream>
#include <strstream>

#include <algorithm>

////////////////////////////////////////////////////////////////////////////////////////////////////

mm1_sequencebuilder mm1_mdl::amino_builder(MM1_AMINO_BUILDER_PATH);
mm1_sequencebuilder mm1_mdl::nucleic_builder(MM1_NUCLEIC_BUILDER_PATH);

mm1_mdl::mm1_mdl(ostream * p1) : bs_mdl_extended(p1), bs_mdl_simple(p1)
{
	nmol = NOT_DEFINED;
	ref_rci_v = NULL;
	
	box_hdim[0] = 1.50;
	box_hdim[1] = 1.50;
	box_hdim[2] = 1.50;
	
	box_fdim[0] = 2.0 * box_hdim[0];
	box_fdim[1] = 2.0 * box_hdim[1];
	box_fdim[2] = 2.0 * box_hdim[2];
}

mm1_mdl::~mm1_mdl(void)
{
}

const char * mm1_mdl::GetProjectFileNameExtension(void)
{
	static const char ext[] = "mm1gp";
	return ext;
}

void mm1_mdl::PushCRDSets(i32u p1)
{
//	cs_vector.resize(cs_vector.size() + p1);			// more convenient way?!?!?!
//	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back();		// more crash-proof way?!?!?!?
	for (i32u n1 = 0;n1 < p1;n1++) cs_vector.push_back(crd_set());	// or even this way?!?!?!?
	
	fGL_a3 newcrd = { 0.0, 0.0, 0.0 };
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32u n1 = 0;n1 < p1;n1++)
		{
			(* it1).crd_vector.push_back(newcrd);
		}
	}
}

void mm1_mdl::PopCRDSets(i32u)
{
}

void mm1_mdl::CopyCRDSet(i32u, i32u)
{
}

void mm1_mdl::SwapCRDSets(i32u, i32u)
{
}

void mm1_mdl::CenterCRDSet(i32u p1)
{
	fGL sum[3] = { 0.0, 0.0, 0.0 };
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s n1 = 0;n1 < 3;n1++) sum[n1] += (* it1).crd_vector[p1][n1];
	}
	
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		for (i32s n1 = 0;n1 < 3;n1++)
		{
			fGL tmp1 = sum[n1] / (f64) atom_list.size();
			(* it1).crd_vector[p1][n1] -= tmp1;
		}
	}
}

void mm1_mdl::ReserveCRDSets(i32u)
{
}

// this is used to import hyperchem HIN-files.
// if "append" is false then a new coordinate set will be created instead of new atoms/bonds ???

bool mm1_mdl::ImportHIN(const char * filename, bool append)
{
	ifstream file(filename, ios::in);
	
	if (file.fail())
	{
		if (ostr != NULL) (* ostr) << "file \"" << filename << "\" not found." << endl;
		return false;
	}
	
	i32s crdset = 0;
	iter_mm1al it1 = atom_list.begin();
	iter_mm1bl it2 = bond_list.begin();
	if (!append)
	{
		crdset = cs_vector.size();
		PushCRDSets(1);
	}
	
	do file.getline(buffer, sizeof(buffer));
	while (buffer[0] != 'v');
	
	char str1[5], str2[5], str3[5], str4[5], str5[5];
	fGL tmp1[4]; i32s tmp2[2]; vector<mm1_atom *> ref;
	
	while (!file.eof())
	{
		if (file.peek() == 'm') ref.resize(0);
		if (file.peek() == 'a')
		{
			file >> str1 >> tmp2[0] >> str2 >> str3 >> str4 >> str5;
			file >> tmp1[3] >> tmp1[0] >> tmp1[1] >> tmp1[2] >> tmp2[1];
			for (i32s n1 = 0;n1 < 3;n1++) tmp1[n1] /= 10.0;
			
			element el = element(str3);
			if (append)
			{
				mm1_atom newatom(el, tmp1, cs_vector.size());
				AddAtom(newatom); ref.push_back(& atom_list.back());
			}
			else
			{
				if (it1 == atom_list.end()) return false;
				if (el.GetAtomicNumber() != (* it1).el.GetAtomicNumber()) return false;
				for (i32s n1 = 0;n1 < 3;n1++) (* it1).crd_vector[crdset][n1] = tmp1[n1];
				ref.push_back(& (* it1)); it1++;
			}
			
			for (i32s n1 = 0;n1 < tmp2[1];n1++)
			{
				file >> tmp2[0] >> str1;
				if (tmp2[0] < (i32s) ref.size())
				{
					bondtype bt;
					switch (str1[0])
					{
						case 's': bt = bondtype('S'); break;
						case 'a': bt = bondtype('C'); break;
						case 'd': bt = bondtype('D'); break;
						case 't': bt = bondtype('T'); break;
					}
					
					if (append)
					{
						mm1_bond newbond(ref[tmp2[0] - 1], ref[ref.size() - 1], bt);
						AddBond(newbond);
					}
					else
					{
						if (it2 == bond_list.end()) return false;
						else if (bt.GetValue() != (* it2).bt.GetValue()) return false;
						else it2++;
					}
				}
			}
		}
		
		file.getline(buffer, sizeof(buffer));
	}
	
	file.close();
	
	CenterCRDSet(crdset);
	cs_vector[crdset].visible = true;
	
	return true;
}

// this is used to import ENT-files from the PDB.
// "chain" should be symbol of the chain to import, or NOT_DEFINED to import all chains.
// the current implementation will ignore HETATMs.

void mm1_mdl::ImportENT(const char * filename, signed char chain)
{
	ifstream file(filename, ios::in);
	
	if (file.fail())
	{
		if (ostr != NULL) (* ostr) << "file \"" << filename << "\" not found." << endl;
		return;
	}
	
	// read the whole file to temporary arrays...
	
	vector<mm1_ent_import1> ssb_data;
	vector<mm1_ent_import2> atom_data;
	
	while (!file.eof())
	{
		char record_id[8];
		for (i32s n1 = 0;n1 < 6;n1++)
		{
			char tchar = (char) file.get();
			if (tchar != ' ') record_id[n1] = tchar;
			else record_id[n1] = 0;
		}
		
		record_id[6] = 0;	// terminate the string...
		
		if (!strcmp("SSBOND", record_id))
		{
			i32s serial_number; file >> serial_number;
			mm1_ent_import1 new_data; new_data.ref = NULL;
			
			for (i32s n1 = 0;n1 < 2;n1++)
			{
				file >> buffer; file.get();
				new_data.chn = (char) file.get();
				
				for (i32s n2 = 0;n2 < 6;n2++) buffer[n2] = (char) file.get();
				istrstream str(buffer, sizeof(buffer));
				str >> new_data.res;
				
				ssb_data.push_back(new_data);
			}
		}
		
		if (!strcmp("ATOM", record_id))
		{
			i32s serial_number; file >> serial_number;
			mm1_ent_import2 new_data; new_data.ref = NULL;
			
			for (i32s n1 = 0;n1 < 18;n1++) buffer[n1] = (char) file.get();
			buffer[5] = ' '; buffer[15] = ' ';
			
			istrstream str(buffer, sizeof(buffer));
			str >> new_data.atm_name >> new_data.res_name; str.get();
			new_data.chn = (char) str.get(); str >> new_data.res;
			
			file >> new_data.crd[0]; new_data.crd[0] /= 10.0;
			file >> new_data.crd[1]; new_data.crd[1] /= 10.0;
			file >> new_data.crd[2]; new_data.crd[2] /= 10.0;
			
			bool test1 = (chain == NOT_DEFINED);
			bool test2 = (new_data.chn == chain);
			if (test1 || test2) atom_data.push_back(new_data);
		}
		
		file.getline(buffer, sizeof(buffer));
	}
	
	if (atom_data.empty())
	{
		cout << "no atoms found !!!" << endl;
		return;
	}
	
	// count the chains...
	
	vector<char> chn_data;
	chn_data.push_back(atom_data.front().chn);
	for (i32u n1 = 1;n1 < atom_data.size();n1++)
	{
		char current_chain = atom_data[n1].chn;
		if (current_chain == chn_data.back()) continue;
		else chn_data.push_back(current_chain);
	}
	
	// and create them, checking validity of the residues...
	
	for (i32u n1 = 0;n1 < chn_data.size();n1++)
	{
		i32s range1[2]; range1[0] = 0; while (atom_data[range1[0]].chn != chn_data[n1]) range1[0]++;
		range1[1] = range1[0]; while (range1[1] < (i32s) atom_data.size() && atom_data[range1[1]].chn == chn_data[n1]) range1[1]++;
		
		vector<i32s> res_data;
		i32s range2[2]; range2[0] = range1[0];
		while (range2[0] < range1[1])
		{
			i32s residue = atom_data[range2[0]].res; const char * res_name = atom_data[range2[0]].res_name;
			range2[1] = range2[0]; while (range2[1] < (i32s) atom_data.size() && atom_data[range2[1]].res == residue) range2[1]++;
			
			bool standard = false;
			if (!strcmp("ALA", res_name)) standard = true;
			if (!strcmp("ARG", res_name)) standard = true;
			if (!strcmp("ASN", res_name)) standard = true;
			if (!strcmp("ASP", res_name)) standard = true;
			if (!strcmp("CYS", res_name)) standard = true;
			if (!strcmp("GLN", res_name)) standard = true;
			if (!strcmp("GLU", res_name)) standard = true;
			if (!strcmp("GLY", res_name)) standard = true;
			if (!strcmp("HIS", res_name)) standard = true;
			if (!strcmp("ILE", res_name)) standard = true;
			if (!strcmp("LEU", res_name)) standard = true;
			if (!strcmp("LYS", res_name)) standard = true;
			if (!strcmp("MET", res_name)) standard = true;
			if (!strcmp("PHE", res_name)) standard = true;
			if (!strcmp("PRO", res_name)) standard = true;
			if (!strcmp("SER", res_name)) standard = true;
			if (!strcmp("THR", res_name)) standard = true;
			if (!strcmp("TRP", res_name)) standard = true;
			if (!strcmp("TYR", res_name)) standard = true;
			if (!strcmp("VAL", res_name)) standard = true;
			
			bool skip = false;
			if (standard)
			{
				if (ImportENT_sub1(atom_data, range2, "N", true) == NOT_DEFINED) skip = true;
				if (ImportENT_sub1(atom_data, range2, "CA", true) == NOT_DEFINED) skip = true;
				if (ImportENT_sub1(atom_data, range2, "C", true) == NOT_DEFINED) skip = true;
				if (ImportENT_sub1(atom_data, range2, "O", true) == NOT_DEFINED) skip = true;
			}
			else
			{
				cout << "could not recognize this residue: " << res_name << endl;
				skip = true;
			}
			
			if (skip)
			{
				cout << "skipping broken residue " << residue << " " << res_name << endl;
				atom_data.erase(atom_data.begin() + range2[0], atom_data.begin() + range2[1]);
				range1[1] += (range2[0] - range2[1]);
			}
			else
			{
				res_data.push_back(range2[0]);
				for (i32s n2 = range2[0];n2 < range2[1];n2++)
				{
					strcpy(buffer, atom_data[n2].atm_name); buffer[1] = 0;
					element el = element(buffer);
					
					if (el.GetAtomicNumber() == 1) continue;
					mm1_atom newatom(el, atom_data[n2].crd, cs_vector.size());
					AddAtom(newatom); atom_data[n2].ref = (& atom_list.back());
				}
				
				ImportENT_sub2(atom_data, range2, "N", "CA", 'S');
				ImportENT_sub2(atom_data, range2, "CA", "C", 'S');
				ImportENT_sub2(atom_data, range2, "C", "O", 'D');
				
				if (!strcmp("ALA", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
				}
				
				if (!strcmp("ARG", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD", 'S');
					ImportENT_sub2(atom_data, range2, "CD", "NE", 'S');
					ImportENT_sub2(atom_data, range2, "NE", "CZ", 'S');
					ImportENT_sub2(atom_data, range2, "CZ", "NH1", 'C');
					ImportENT_sub2(atom_data, range2, "CZ", "NH2", 'C');
				}
				
				if (!strcmp("ASN", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "OD1", 'D');
					ImportENT_sub2(atom_data, range2, "CG", "ND2", 'S');
				}
				
				if (!strcmp("ASP", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "OD1", 'C');
					ImportENT_sub2(atom_data, range2, "CG", "OD2", 'C');
				}
				
				if (!strcmp("CYS", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "SG", 'S');
				}
				
				if (!strcmp("GLN", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD", 'S');
					ImportENT_sub2(atom_data, range2, "CD", "OE1", 'D');
					ImportENT_sub2(atom_data, range2, "CD", "NE2", 'S');
				}
				
				if (!strcmp("GLU", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD", 'S');
					ImportENT_sub2(atom_data, range2, "CD", "OE1", 'C');
					ImportENT_sub2(atom_data, range2, "CD", "OE2", 'C');
				}
				
				// GLY -> we don't have to do anything in this case...
				// GLY -> we don't have to do anything in this case...
				// GLY -> we don't have to do anything in this case...
				
				if (!strcmp("HIS", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "ND1", 'C');
					ImportENT_sub2(atom_data, range2, "CG", "CD2", 'C');
					ImportENT_sub2(atom_data, range2, "ND1", "CE1", 'C');
					ImportENT_sub2(atom_data, range2, "CD2", "NE2", 'C');
					ImportENT_sub2(atom_data, range2, "CE1", "NE2", 'C');
				}
				
				if (!strcmp("ILE", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG1", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG2", 'S');
					ImportENT_sub2(atom_data, range2, "CG1", "CD1", 'S');
				}
				
				if (!strcmp("LEU", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD1", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD2", 'S');
				}
				
				if (!strcmp("LYS", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD", 'S');
					ImportENT_sub2(atom_data, range2, "CD", "CE", 'S');
					ImportENT_sub2(atom_data, range2, "CE", "NZ", 'S');
				}
				
				if (!strcmp("MET", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "SD", 'S');
					ImportENT_sub2(atom_data, range2, "SD", "CE", 'S');
				}
				
				if (!strcmp("PHE", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD1", 'C');
					ImportENT_sub2(atom_data, range2, "CG", "CD2", 'C');
					ImportENT_sub2(atom_data, range2, "CD1", "CE1", 'C');
					ImportENT_sub2(atom_data, range2, "CD2", "CE2", 'C');
					ImportENT_sub2(atom_data, range2, "CE1", "CZ", 'C');
					ImportENT_sub2(atom_data, range2, "CE2", "CZ", 'C');
				}
				
				if (!strcmp("PRO", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD", 'S');
					ImportENT_sub2(atom_data, range2, "CD", "N", 'S');
				}
				
				if (!strcmp("SER", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "OG", 'S');
				}
				
				if (!strcmp("THR", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "OG1", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG2", 'S');
				}
				
				if (!strcmp("TRP", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD1", 'C');
					ImportENT_sub2(atom_data, range2, "CG", "CD2", 'C');
					ImportENT_sub2(atom_data, range2, "CD1", "NE1", 'C');
					ImportENT_sub2(atom_data, range2, "CD2", "CE2", 'C');
					ImportENT_sub2(atom_data, range2, "CD2", "CE3", 'C');
					ImportENT_sub2(atom_data, range2, "NE1", "CE2", 'C');
					ImportENT_sub2(atom_data, range2, "CE2", "CZ2", 'C');
					ImportENT_sub2(atom_data, range2, "CE3", "CZ3", 'C');
					ImportENT_sub2(atom_data, range2, "CZ2", "CH2", 'C');
					ImportENT_sub2(atom_data, range2, "CZ3", "CH2", 'C');
				}
				
				if (!strcmp("TYR", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG", 'S');
					ImportENT_sub2(atom_data, range2, "CG", "CD1", 'C');
					ImportENT_sub2(atom_data, range2, "CG", "CD2", 'C');
					ImportENT_sub2(atom_data, range2, "CD1", "CE1", 'C');
					ImportENT_sub2(atom_data, range2, "CD2", "CE2", 'C');
					ImportENT_sub2(atom_data, range2, "CE1", "CZ", 'C');
					ImportENT_sub2(atom_data, range2, "CE2", "CZ", 'C');
					ImportENT_sub2(atom_data, range2, "CZ", "OH", 'S');
				}
				
				if (!strcmp("VAL", res_name))
				{
					ImportENT_sub2(atom_data, range2, "CA", "CB", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG1", 'S');
					ImportENT_sub2(atom_data, range2, "CB", "CG2", 'S');
				}
				
				range2[0] = range2[1];
			}
		}
		
		// add connecting bonds between residues...
		
		res_data.push_back(range1[1]);
		for (i32s n2 = 0;n2 < ((i32s) res_data.size()) - 2;n2++)
		{
			i32s r1[2] = { res_data[n2],  res_data[n2 + 1] };
			i32s r2[2] = { res_data[n2 + 1],  res_data[n2 + 2] };
			
			i32s ind1 = ImportENT_sub1(atom_data, r1, "C", false);
			i32s ind2 = ImportENT_sub1(atom_data, r2, "N", false);
			
			if (ind1 == NOT_DEFINED || ind2 == NOT_DEFINED)
			{
				(* ostr) << "could not connect ..." << endl;
				exit(EXIT_FAILURE);
			}
			else
			{
				bondtype bt = bondtype('S');
				mm1_bond newbond(atom_data[ind1].ref, atom_data[ind2].ref, bt);
				AddBond(newbond);
			}
		}
		
		// check the c-terminal residue...
		
		if (res_data.size() < 2) continue;
		i32s last[2] = { res_data[res_data.size() - 2], res_data[res_data.size() - 1] };
		
		i32s ind1 = ImportENT_sub1(atom_data, last, "C", false);
		i32s ind2 = ImportENT_sub1(atom_data, last, "O", false);
		
		if (ind1 == NOT_DEFINED || ind2 == NOT_DEFINED)
		{
			(* ostr) << "could not find c-term group..." << endl;
			exit(EXIT_FAILURE);
		}
		else
		{
			bondtype bt = bondtype('C');
			mm1_bond tmpbond(atom_data[ind1].ref, atom_data[ind2].ref, bt);
			iter_mm1bl it1 = find(bond_list.begin(), bond_list.end(), tmpbond);
			if (it1 != bond_list.end()) (* it1).bt = bt;
			
			ind2 = ImportENT_sub1(atom_data, last, "OXT", false);
			if (ind2 != NOT_DEFINED)
			{
				mm1_bond newbond(atom_data[ind1].ref, atom_data[ind2].ref, bt);
				AddBond(newbond);
			}
			else
			{
				cout << "missing terminal oxygen..." << endl;
				
				i32s ind[3];
				ind[0] = ImportENT_sub1(atom_data, last, "CA", false);
				ind[1] = ImportENT_sub1(atom_data, last, "C", false);
				ind[2] = ImportENT_sub1(atom_data, last, "O", false);
				
				v3d<fGL> v1 = v3d<fGL>(atom_data[ind[1]].ref->crd_vector[0].data, atom_data[ind[0]].ref->crd_vector[0].data);
				v3d<fGL> v2 = v3d<fGL>(atom_data[ind[1]].ref->crd_vector[0].data, atom_data[ind[2]].ref->crd_vector[0].data);
				fGL tmp2 = v1.len(); fGL tmp3 = v1.spr(v2) / (tmp2 * tmp2);
				v1 = v1 * tmp3; v2 = v2 - v1;
				
				fGL tmp4[3];
				for (i32s n1 = 0;n1 < 3;n1++)
				{
					tmp4[n1] = atom_data[ind[1]].ref->crd_vector[0][n1];
					tmp4[n1] += v1.data[n1] - v2.data[n1];
				}
				
				element el = element(8);
				mm1_atom newatom(el, tmp4, cs_vector.size());
				AddAtom(newatom);
				
				mm1_bond newbond(atom_data[ind1].ref, (& atom_list.back()), bt);
				AddBond(newbond);
			}
		}
	}
	
	// add disulphide bonds...
	
	for (i32u n1 = 0;n1 < ssb_data.size();n1++)
	{
		i32u counter = 0;
		while (counter < atom_data.size())
		{
			bool test1 = (ssb_data[n1].chn == atom_data[counter].chn);
			bool test2 = (ssb_data[n1].res == atom_data[counter].res);
			bool test3 = !strcmp(atom_data[counter].atm_name, "SG");
			if (test1 && test2 && test3) break; else counter++;
		}
		
		if (counter == atom_data.size()) continue;
		ssb_data[n1].ref = atom_data[counter].ref;
	}
	
	for (i32u n1 = 0;n1 < ssb_data.size();n1 += 2)
	{
		if (ssb_data[n1 + 0].ref == NULL || ssb_data[n1 + 1].ref == NULL)
		{
			cout << "could not create bridge ";
			cout << ssb_data[n1 + 0].chn << " " << ssb_data[n1 + 0].res << " -> ";
			cout << ssb_data[n1 + 1].chn << " " << ssb_data[n1 + 1].res << endl;
		}
		else
		{
			bondtype bt = bondtype('S');
			mm1_bond newbond(ssb_data[n1 + 0].ref, ssb_data[n1 + 1].ref, bt);
			AddBond(newbond);
		}
	}
	
	CenterCRDSet(0);
	cout << "done." << endl;
}

i32s mm1_mdl::ImportENT_sub1(vector<mm1_ent_import2> & atom_data, i32s * range, const char * atom_name, bool flag)
{
	for (i32s n1 = range[0];n1 < range[1];n1++)
	{
		if (!strcmp(atom_data[n1].atm_name, atom_name)) return n1;
	}
	
	if (flag && ostr != NULL) (* ostr) << "atom " << atom_name << " is missing..." << endl;
	return NOT_DEFINED;
}

void mm1_mdl::ImportENT_sub2(vector<mm1_ent_import2> & atom_data, i32s * range, const char * at1, const char * at2, char btype)
{
	i32s ind1 = ImportENT_sub1(atom_data, range, at1, false);
	i32s ind2 = ImportENT_sub1(atom_data, range, at2, false);
	
	if (ind1 == NOT_DEFINED || ind2 == NOT_DEFINED) return;
	
	bondtype bt = bondtype(btype);
	mm1_bond newbond(atom_data[ind1].ref, atom_data[ind2].ref, bt);
	AddBond(newbond);
}

// just push new atom to the atom list and update colors if necessary.
// mark for updated connectivity (for GUI's and "save") ??????????

void mm1_mdl::AddAtom(mm1_atom & p1)
{
	InvalidateGroups();
	atom_list.push_back(p1);
}

// delete all bonds associated with this atom, and erase atom from the list...

void mm1_mdl::RemoveAtom(iter_mm1al it1)
{
	InvalidateGroups();
	
	// this strange while-loop is needed because removing bonds
	// will invalidate all those atom::cr_list-iterators...
	
	while ((* it1).cr_list.begin() != (* it1).cr_list.end())
	{
		mm1_cr * ref = & (* it1).cr_list.back();
		iter_mm1bl it2 = find(bond_list.begin(), bond_list.end(), (* ref->bndr));
		
		if (it2 != bond_list.end()) RemoveBond(it2);
		else if (ostr != NULL) (* ostr) << "bad news: find failed in RA" << endl;
	}
	
	atom_list.erase(it1);
}

// add infos for both atoms and push new bond to the bond list.

void mm1_mdl::AddBond(mm1_bond & p1)
{
	InvalidateGroups();
	bond_list.push_back(p1);
	
	mm1_cr info1 = mm1_cr(p1.atmr[1], & bond_list.back());
	p1.atmr[0]->cr_list.push_back(info1);
	
	mm1_cr info2 = mm1_cr(p1.atmr[0], & bond_list.back());
	p1.atmr[1]->cr_list.push_back(info2);
}

// remove infos from the atoms and erase bond from the bond list.

void mm1_mdl::RemoveBond(iter_mm1bl it1)
{
	InvalidateGroups();
	mm1_cr tmpinfo = mm1_cr(NULL, & (* it1));
	iter_mm1cl it2;
	
	it2 = find((* it1).atmr[0]->cr_list.begin(), (* it1).atmr[0]->cr_list.end(), tmpinfo);
	if (it2 != (* it1).atmr[0]->cr_list.end()) (* it1).atmr[0]->cr_list.erase(it2);
	else if (ostr != NULL) (* ostr) << "bad news: find #1 failed in RB" << endl;
	
	it2 = find((* it1).atmr[1]->cr_list.begin(), (* it1).atmr[1]->cr_list.end(), tmpinfo);
	if (it2 != (* it1).atmr[1]->cr_list.end()) (* it1).atmr[1]->cr_list.erase(it2);
	else if (ostr != NULL) (* ostr) << "bad news: find #2 failed in RB" << endl;
	
	bond_list.erase(it1);
}

void mm1_mdl::ReadStream(istream & p1)
{
	vector<mm1_atom *> tmp1; i32s tmp2[2];
	char tmp3; fGL tmp4[3];
	
	while (p1.peek() != 'E')
	{
		p1 >> tmp2[0] >> tmp2[1];
		p1 >> tmp4[0] >> tmp4[1] >> tmp4[2] >> tmp2[0]; p1.get();
		
		element el = element(tmp2[1]);
		mm1_atom newatom(el, tmp4, cs_vector.size());
		AddAtom(newatom); tmp1.push_back(& atom_list.back());
		
		for (i32s n1 = 0;n1 < tmp2[0];n1++)
		{
			p1 >> tmp2[1] >> tmp3;
			if (tmp2[1] < (i32s) tmp1.size())
			{
				mm1_bond newbond(tmp1[tmp2[1]], tmp1[tmp1.size() - 1], bondtype(tmp3));
				AddBond(newbond);
			}
		}
		
		p1.getline(buffer, sizeof(buffer));
	}
}

// this fileformat is not documented yet... there is not even any identifier
// or any record about the version -> it might be difficult to read these files later.

void mm1_mdl::WriteStream(ostream & p1)
{
	UpdateIndex();
	
	iter_mm1al it1 = atom_list.begin();
	while (it1 != atom_list.end())
	{
		p1 << (* it1).index << " " << (* it1).el.GetAtomicNumber() << " ";
		p1 << (* it1).crd_vector[0][0] << " " << (* it1).crd_vector[0][1] << " " << (* it1).crd_vector[0][2] << " ";
		
		vector<i32s> tmp1;	// this is for atomic numbers...
		vector<char> tmp2;	// this is for bond type symbols...
		
		iter_mm1cl it2;
		for (it2 = (* it1).cr_list.begin();it2 != (* it1).cr_list.end();it2++)
		{
			tmp1.push_back((* it2).atmr->index);
			tmp2.push_back((* it2).bndr->bt.GetSymbol1());
		}
		
		p1 << tmp1.size() << ": "; it1++;
		for (i32u n1 = 0;n1 < tmp1.size();n1++)
		{
			p1 << tmp1[n1] << " ";
			p1 << tmp2[n1] << " ";
		}
		
		p1 << endl;
	}
	
	p1 << "END" << endl;
}

// currently we use tetrahedral SP3 default geometries here in all cases, and this works only
// for a few elements... random numbers are used, but not properly initialized ?!?!?!

void mm1_mdl::AddHydrogens(void)
{
	for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		float n1; float n2 = 0;
		switch ((* it1).el.GetAtomicNumber())
		{
			case 6:		n1 = 4.0; break;
			case 7:		n1 = 3.0; break;
			case 8:		n1 = 2.0; break;
			default:	n1 = -1.0;
		}
		
		if (n1 < 0.0) continue;
		
		vector<mm1_atom *> ref;
		for (iter_mm1bl it2 = bond_list.begin();it2 != bond_list.end();it2++)
		{
			if ((* it2).atmr[0] == & (* it1)) ref.push_back((* it2).atmr[1]);
			else if ((* it2).atmr[1] == & (* it1)) ref.push_back((* it2).atmr[0]);
			else continue;
			
			switch ((* it2).bt.GetValue())
			{
				case BT_SINGLE:		n2 += 1.0; break;
				case BT_CNJGTD:		n2 += 1.5; break;
				case BT_DOUBLE:		n2 += 2.0; break;
				case BT_TRIPLE:		n2 += 3.0; break;
			}
		}
		
		if ((n1 - 0.1) < n2) continue;
		
		v3d<fGL> v1, v2, v3;
		const fGL ideal = acos(-1.0 / 3.0);
		fGL rnd[3] =
		{
			(fGL) rand() / (fGL) RAND_MAX - 0.5,
			(fGL) rand() / (fGL) RAND_MAX - 0.5,
			(fGL) rand() / (fGL) RAND_MAX - 0.5
		};
		
		switch (ref.size())
		{
			case 0:
			v1 = v3d<fGL>(rnd);
			v1 = v1 * (0.11 / v1.len());
			break;
			
			case 1:
			v1 = v3d<fGL>(rnd);
			v2 = v3d<fGL>((* it1).crd_vector[0].data, ref[0]->crd_vector[0].data);
			v3 = v1.vpr(v2);
			v2 = v2 * (0.11 * cos(ideal) / v2.len());
			v3 = v3 * (0.11 * sin(ideal) / v3.len());
			v1 = v2 + v3;
			break;
			
			case 2:
			v1 = v3d<fGL>((* it1).crd_vector[0].data, ref[0]->crd_vector[0].data);
			v2 = v3d<fGL>((* it1).crd_vector[0].data, ref[1]->crd_vector[0].data);
			v1 = v1 * (1.0 / v1.len());
			v2 = v2 * (1.0 / v2.len());
			v2 = v1 + v2; v3 = v1.vpr(v2);
			v2 = v2 * (0.11 * cos(ideal) / v2.len());
			v3 = v3 * (0.11 * sin(ideal) / v3.len());
			v1 = v2 + v3;
			break;
			
			case 3:
			v1 = v3d<fGL>((* it1).crd_vector[0].data, ref[0]->crd_vector[0].data);
			v2 = v3d<fGL>((* it1).crd_vector[0].data, ref[1]->crd_vector[0].data);
			v3 = v3d<fGL>((* it1).crd_vector[0].data, ref[2]->crd_vector[0].data);
			v1 = v1 * (1.0 / v1.len());
			v2 = v2 * (1.0 / v2.len());
			v3 = v3 * (1.0 / v3.len());
			v1 = v1 + v2; v1 = v1 + v3;
			v1 = v1 * (-0.11 / v1.len());
			break;
		}
		
		fGL tmp[3]; element el = element(1);
		for (i32s n3 = 0;n3 < 3;n3++) tmp[n3] = (* it1).crd_vector[0][n3] + v1.data[n3];
		mm1_atom newatom(el, tmp, cs_vector.size());
		AddAtom(newatom);
		
		bondtype single = bondtype('S');
		mm1_bond newbond(& (* it1), & atom_list.back(), single);
		AddBond(newbond);
		
		// we have to cycle this several times, since we might want
		// to add several hydrogens. change that for-loop -> while-loop ???
		
		it1--;
	}
}

void mm1_mdl::RemoveHydrogens(void)
{
	iter_mm1bl it1 = bond_list.begin();
	while (it1 != bond_list.end())
	{
		bool flag = false;
		if ((* it1).atmr[0]->el.GetAtomicNumber() == 1) flag = true;
		if ((* it1).atmr[1]->el.GetAtomicNumber() == 1) flag = true;
		iter_mm1bl it2 = it1++; if (flag) RemoveBond(it2);
	}
	
	iter_mm1al it2 = atom_list.begin();
	while (it2 != atom_list.end())
	{
		iter_mm1al it3 = it2++;
		if ((* it3).el.GetAtomicNumber() == 1) RemoveAtom(it3);
	}
}

// this will group atom list so that molecules (and chains/residues if defined) will form
// continuous groups. will validate mm1_model::nmol and permit the use of mm1_model::GetRange()-functions.

void mm1_mdl::GatherGroups(void)
{
	InvalidateGroups(); iter_mm1al it1;
	nmol = 0; ref_rci_v = new vector<mm1_chn_info *>;
	
	for (it1 = atom_list.begin();it1 != atom_list.end();it1++)
	{
		(* it1).id[0] = (* it1).id[1] = (* it1).id[2] = (* it1).id[3] = NOT_DEFINED;
	}
	
	while (true)
	{
		for (it1 = atom_list.begin();it1 != atom_list.end() && (* it1).id[0] != NOT_DEFINED;it1++);
		if (it1 != atom_list.end()) GatherAtoms(& (* it1), nmol++); else break;
	}
	
	atom_list.sort();
}

// this is just default version of GetRange() using the full range of atom list iterators...

void mm1_mdl::GetRange(i32s ind, i32s value, iter_mm1al * result)
{
	iter_mm1al range[2] = { atom_list.begin(), atom_list.end() };
	GetRange(ind, range, value, result);
}

// this will reduce the initial range of two atom list iterators to some subrange
// with certain values in certain mm1_atom::id[]-fields. before using this you MUST
// call mm1_model::GatherGroups() to arrange the atom list !!!

void mm1_mdl::GetRange(i32s ind, iter_mm1al * range, i32s value, iter_mm1al * result)
{
	result[0] = range[0]; while (result[0] != range[1] && (* result[0]).id[ind] != value) result[0]++;
	result[1] = result[0]; while (result[1] != range[1] && (* result[1]).id[ind] == value) result[1]++;
}

// adding/removing atoms/bonds can split grouping so when that happens this function
// should be called to discard all grouping information...

void mm1_mdl::InvalidateGroups(void)
{
	nmol = NOT_DEFINED;
	if (ref_rci_v != NULL)
	{
		vector<mm1_chn_info *> & rci_vector = (* ref_rci_v);
		for (i32u n1 = 0;n1 < rci_vector.size();n1++) delete rci_vector[n1];
		delete ref_rci_v; ref_rci_v = NULL;
	}
}

i32s mm1_mdl::FindPath(mm1_atom * ref1, mm1_atom * ref2, i32s max, i32s flag, i32s dist)
{
	if (ref1 == ref2) return dist;
	if (dist == max) return NOT_FOUND;
	
	i32s tmp1 = NOT_FOUND; iter_mm1cl it1;
	for (it1 = ref1->cr_list.begin();it1 != ref1->cr_list.end();it1++)
	{
		if ((* it1).bndr->flags[flag]) continue;
		
		(* it1).bndr->flags[flag] = true;
		i32s tmp2 = FindPath((* it1).atmr, ref2, max, flag, dist + 1);
		(* it1).bndr->flags[flag] = false; if (tmp2 < tmp1) tmp1 = tmp2;
	}
	
	return tmp1;
}

bool mm1_mdl::FindRing(mm1_atom * ref1, mm1_atom * ref2, signed char * str, i32s size, i32s flag, i32s dist)
{
	static vector<signed char> ring_vector;
		
	if (!dist && str != NULL) ring_vector.resize(0);
	else if (dist && ref1 == ref2)
	{
		if (dist != size) return false;
		
		if (str != NULL)
		{
			for (i32u n1 = 0;n1 < strlen((const char *) str);n1++)
			{
				if (!(n1 % 2) && str[n1] == '?') continue;
				if ((n1 % 2) && str[n1] == NOT_DEFINED) continue;
				if (str[n1] != ring_vector[n1]) return false;
			}
		}
		
		return true;
	}
	
	if (dist == size) return false; iter_mm1cl it1;
	for (it1 = ref1->cr_list.begin();it1 != ref1->cr_list.end();it1++)
	{
		if ((* it1).bndr->flags[flag]) continue;
		
		if (str != NULL)
		{
			ring_vector.push_back((* it1).bndr->bt.GetSymbol2());
			ring_vector.push_back((signed char) (* it1).atmr->el.GetAtomicNumber());
		}
		
		(* it1).bndr->flags[flag] = true;
		bool result = FindRing((* it1).atmr, ref2, str, size, flag, dist + 1);
		(* it1).bndr->flags[flag] = false;
		
		if (str != NULL)
		{
			ring_vector.pop_back();
			ring_vector.pop_back();
		}
		
		if (result) return true;
	}
	
	return false;
}

void mm1_mdl::UpdateIndex(void)
{
	i32s counter = 0;
	iter_mm1al it1 = atom_list.begin();
	while (it1 != atom_list.end())
	{
		(* it1++).index = counter++;
	}
}

// this will set molecule numbers quickly using a recursive search algorithm.
// this is private because only mm1_model::GatherGroups() should use this...

void mm1_mdl::GatherAtoms(mm1_atom * ref, i32s id)
{
	if (ref->id[0] != NOT_DEFINED) return;
	ref->id[0] = id; iter_mm1cl it1 = ref->cr_list.begin();
	while (it1 != ref->cr_list.end()) GatherAtoms((* it1++).atmr, id);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
