// QM2ENG1.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#ifdef ENABLE_MPQC
#include "qm2eng1.h"

#include <strstream>

#include "util/misc/regtime.h"			// RegionTimer
#include "util/misc/formio.h"			// SCFormIO

#include "math/scmat/vector3.h"			// SCVector3

#include "chemistry/molecule/formula.h"		// MolecularFormula

// force linkages!!! this will make sure that all important stuff will be linked from the libraries...
// force linkages!!! this will make sure that all important stuff will be linked from the libraries...
// force linkages!!! this will make sure that all important stuff will be linked from the libraries...

#include "chemistry/qc/scf/linkage.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

i32u qm2_eng_mpqc::ifile_name_counter = 0;

qm2_eng_mpqc::qm2_eng_mpqc(qm2_mdl & p1) : qm2_eng(p1)
{
	char filename[32];
	GetInputFileName(filename, sizeof(filename));
	
	cout << "writing MPQC-input file " << filename << endl;
	
	ofstream file(filename, ios::out);
	WriteInputFile(file);
	file.close();
	
	// now we start working with MPQC classes...
	// now we start working with MPQC classes...
	// now we start working with MPQC classes...
	
	// we should call here first ExEnv::init() and pass the command-line arguments to MPQC.
	// however, it seems that we can skip this if we are not doing parallel processing...
	
	grp = MessageGrp::get_default_messagegrp();
	cout << "using " << grp->class_name() << " as MessageGroup..." << endl;
	
	thread = ThreadGrp::get_default_threadgrp();
	cout << "using " << thread->class_name() << " as ThreadGroup..." << endl;
	
	parsedkv = new ParsedKeyVal(filename);
	keyval = new PrefixKeyVal("mpqc", parsedkv.pointer());
	
	static char basename[] = "ghemical";
	SCFormIO::set_default_basename(basename);
	SCFormIO::set_printnode(0);
	
	if (keyval->exists("matrixkit"))
	{
		SCMatrixKit::set_default_matrixkit(keyval->describedclassvalue("matrixkit"));
	}
	
	mole = keyval->describedclassvalue("mole");
	obwfn = mole; wfn = mole;
	
	MolecularFormula mf(mole->molecule());
	cout << "molecular formula = " << mf.formula() << endl;
}

qm2_eng_mpqc::~qm2_eng_mpqc(void)
{
	MessageGrp::set_default_messagegrp(0);
	ThreadGrp::set_default_threadgrp(0);
	
	SCMatrixKit::set_default_matrixkit(0);
	RegionTimer::set_default_regiontimer(0);
}

void qm2_eng_mpqc::GetInputFileName(char * fn, i32u maxlen)
{
	ostrstream str(fn, maxlen);
	str << "/tmp/mpqc" << setfill('0') << setw(4) << ifile_name_counter++ << ".in" << ends;
}

void qm2_eng_mpqc::WriteInputFile(ofstream & file)
{
	file.setf(ios::fixed);
	
	// molecule-record...
	// molecule-record...
	// molecule-record...
	
	file << "molecule<Molecule>: (" << endl;
	file << "  symmetry = C1" << endl;
	file << "  unit = angstrom" << endl;
	file << "  { atoms geometry } = {" << endl;
	
	// we will write the first coordinate set here by default...
	
	for (iter_qm2al it1 = GetModel()->atom_list.begin();it1 != GetModel()->atom_list.end();it1++)
	{
		fGL * defcrd = (* it1).crd_vector[0].data;
		const fGL cf = 10.0;	// conversion factor for [nm] -> []
		
		file << "    " << (* it1).el.GetSymbol() << "\t[ ";
		file << setprecision(6) << setw(12) << (defcrd[0] * cf) << " ";
		file << setprecision(6) << setw(12) << (defcrd[1] * cf) << " ";
		file << setprecision(6) << setw(12) << (defcrd[2] * cf) << " ";
		file << "]" << endl;
	}
	
	file << "  }" << endl;
	file << ")" << endl;
	
	// basis-record...
	// basis-record...
	// basis-record...
	
	file << "basis<GaussianBasisSet>: (" << endl;
	file << "  name = \"STO-3G\"" << endl;
	file << "  molecule = $:molecule" << endl;
	file << ")" << endl;
	
	// mpqc-record...
	// mpqc-record...
	// mpqc-record...
	
	file << "mpqc: (" << endl;
	file << "  mole<CLHF>: (" << endl;
	file << "    molecule = $:molecule" << endl;
	file << "    basis = $:basis" << endl;
	file << "  )" << endl;
	file << ")" << endl;
}

const f64 * qm2_eng_mpqc::GetCRD(i32u ind)
{
	return NULL;
}

void qm2_eng_mpqc::PutCRD(i32u ind, const f64 * ref)
{
}

// it seems that we get the energy in atomic units... a set of unit conversion functions should
// be made that makes conversions si-units <-> atomic units <-> whatever units. these engine classes
// should have some "states" which determine units that are printed out ?!?!?!?!?!?

void qm2_eng_mpqc::Compute(i32s p1)
{
	energy = mole->energy();
	
	// we should grab the natural orbitals now, I guess...
	// haven't studied it yet in detail but I hope it's correct...
	
//	natorb = wfn->natural_orbitals();
}

void qm2_eng_mpqc::Check(i32s)
{
}

fGL qm2_eng_mpqc::GetESP(fGL * pp, fGL * dd)
{
	// PointChargeData
	// Integral::point_charge -> OneBodyInt
	// what next????? how to evaluate it?????
	
	return 0.0;
}

fGL qm2_eng_mpqc::GetElDens(fGL * pp, fGL * dd)
{
	const double cnv = 18.897162;	// [nm] -> [bohr] conversion factor
	SCVector3 v1((double) pp[0] * cnv, (double) pp[1] * cnv, (double) pp[2] * cnv);
	
	if (dd != NULL)
	{
		double dd2[3];
		double value = wfn->density_gradient(v1, dd2);
		for (i32s n1 = 0;n1 < 3;n1++) dd[n1] = dd2[n1];
		return value;
	}
	else return wfn->density(v1);
}

// natural orbitals seem to be much faster, but I'm not familiar with those...
// natural orbitals seem to be much faster, but I'm not familiar with those...
// natural orbitals seem to be much faster, but I'm not familiar with those...

fGL qm2_eng_mpqc::GetOrbital(fGL * pp, fGL * dd)
{
	const double cnv = 18.897162;	// [nm] -> [bohr] conversion factor
	SCVector3 v1((double) pp[0] * cnv, (double) pp[1] * cnv, (double) pp[2] * cnv);
	
	fGL value = obwfn->orbital(v1, GetModel()->current_orbital);
//	fGL value = wfn->orbital(v1, GetModel()->current_orbital, natorb);	// update "natorb"!!!
	
	if (dd != NULL)		// numerical gradient...
	{
		fGL old;
		const fGL delta = 0.0001;
		
		old = pp[0]; pp[0] += delta;
		dd[0] = (GetOrbital(pp, NULL) - value) / delta;
		pp[0] = old;
		
		old = pp[1]; pp[1] += delta;
		dd[1] = (GetOrbital(pp, NULL) - value) / delta;
		pp[1] = old;
		
		old = pp[2]; pp[2] += delta;
		dd[2] = (GetOrbital(pp, NULL) - value) / delta;
		pp[2] = old;
	}
	
	return value;
}

fGL qm2_eng_mpqc::GetOrbDens(fGL * pp, fGL * dd)
{
	const double cnv = 18.897162;	// [nm] -> [bohr] conversion factor
	SCVector3 v1((double) pp[0] * cnv, (double) pp[1] * cnv, (double) pp[2] * cnv);
	
	fGL value = obwfn->orbital_density(v1, GetModel()->current_orbital);
//	fGL value = wfn->orbital_density(v1, GetModel()->current_orbital, natorb);	// update "natorb"!!!

	if (dd != NULL)		// numerical gradient...
	{
		fGL old;
		const fGL delta = 0.0001;
		
		old = pp[0]; pp[0] += delta;
		dd[0] = (GetOrbDens(pp, NULL) - value) / delta;
		pp[0] = old;
		
		old = pp[1]; pp[1] += delta;
		dd[1] = (GetOrbDens(pp, NULL) - value) / delta;
		pp[1] = old;
		
		old = pp[2]; pp[2] += delta;
		dd[2] = (GetOrbDens(pp, NULL) - value) / delta;
		pp[2] = old;
	}
	
	return value;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_MPQC

// eof
