// MM1GUI.H : a graphical user interface for mm1-models.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is target-dependent...

#ifndef MM1GUI_H
#define MM1GUI_H

#ifdef ENABLE_GUI

class mm1_gui;

class mm1_cm_element;		// mm1_atom *
class mm1_cm_state;		// mm1_atom *

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "guibase.h"

#include "mm1mdl.h"
#include "mm1alg.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

class mm1_gui : public bs_gui, public mm1_mdl
{
	public:
	
	static mm1_cm_element cm_element;
	static mm1_cm_state cm_state;
	
	mm1_atom * draw_data[2];
	
	bool draw_box;
	bool draw_images;
	
	public:
	
	mm1_gui(window_factory &, ostream *);
	virtual ~mm1_gui(void);
	
	const char * GetType(void);			// virtual
	color_mode * GetDefaultColorMode(void);		// virtual
	
	void SelectAll(void);				// virtual
	void InvertSelection(void);			// virtual
	
	bool TestAtom(mm1_atom *, rmode);
	bool TestBond(mm1_bond *, rmode);
	
	void SetColor(color_mode *, mm1_atom *);
	
	void InitGL(void);				// virtual
	void Render(view_window *, rmode);		// virtual
	
	void RenderScene(view_window *, rmode, bool);
	
	void Center(transformer *);			// virtual
	void Transform(transformer *);			// virtual
	
	void DrawEvent(view_window *, vector<iGLu> &);		// virtual
	void EraseEvent(view_window *, vector<iGLu> &);		// virtual
	void SelectEvent(view_window *, vector<iGLu> &);	// virtual
	
	void DoEnergy(void);
	virtual void EnergyReport(mm1_eng *) = 0;
	
	void DoGeomOpt(void);
	virtual bool GeomOptGetParam(mm1_geomopt_param *) = 0;
	
	void DoMolDyn(void);
	virtual bool MolDynGetParam(mm1_moldyn_param *) = 0;
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class mm1_cm_element : public color_mode
{
	protected:
	
	public:
	
	mm1_cm_element(void) : color_mode() { }
	~mm1_cm_element(void) { }
	
	void GetColor(const void *, fGL_a3 &);
};

class mm1_cm_state : public color_mode
{
	protected:
	
	public:
	
	mm1_cm_state(void) : color_mode() { }
	~mm1_cm_state(void) { }
	
	void GetColor(const void *, fGL_a3 &);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

fGL mm1_GetESPValue(fGL *, mm1_gui *, fGL *);
fGL mm1_GetSASValue(fGL *, mm1_gui *, fGL *);

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI
#endif	// MM1GUI_H

// eof
