// PLANE.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "plane.h"

#ifdef ENABLE_GUI
#include "guibase.h"

#include <fstream>
#include <strstream>

////////////////////////////////////////////////////////////////////////////////////////////////////

color_plane::color_plane(const object_location & p1, cp_param & p2) : smart_object(p1)
{
	transparent = p2.transparent;
	
	ostrstream str(name, sizeof(name));
	str << p2.descr << "plane" << ends;
	
	np = p2.np;
	gui = p2.gui;
	
	ref = p2.ref;
	GetValue = p2.vf;
	GetColor = p2.cf;
	value = p2.value;
	
	comp1 = new fGL[np * np];
	comp2 = new fGL[np * np];
	comp3 = new fGL[np * np];
	dist = new fGL[np];
	
	list_id = gui->GetDisplayListIDs(1);
	SetDimension(p2.dim); Update();
}

color_plane::~color_plane(void)
{
	gui->DeleteDisplayLists(list_id, 1);
	
	delete[] comp1; delete[] comp2;
	delete[] comp3; delete[] dist;
}

void color_plane::SetDimension(fGL p1)
{
	dim = p1; fGL tmp1 = np - 1;
	for (i32s n1 = 0;n1 < np;n1++)
	{
		fGL tmp2 = (fGL) n1 / tmp1;
		dist[n1] = dim * (tmp2 - 0.5);
	}
}

void color_plane::GetCRD(i32s * p1, fGL * p2)
{
	for (i32s n1 = 0;n1 < 3;n1++)
	{
		fGL tmp1 = xdir[n1] * dist[p1[0]];
		fGL tmp2 = GetLocData()->ydir[n1] * dist[p1[1]];
		p2[n1] = GetLocData()->crd[n1] + tmp1 + tmp2;
	}
}

void color_plane::OrbitObject(const fGL * p1, const camera & p2)
{
	dummy_object::RotateObject(p1, p2); Update();
}

void color_plane::RotateObject(const fGL * p1, const camera & p2)
{
	dummy_object::OrbitObject(p1, p2); Update();
}

void color_plane::TranslateObject(const fGL * p1, const obj_loc_data * p2)
{
	fGL tmp1[3] = { p1[0], p1[1], p1[2] };
	
	if (p2 == GetLocData())
	{
		tmp1[0] = -tmp1[0];
		tmp1[2] = -tmp1[2];
	}
	
	dummy_object::TranslateObject(tmp1, p2); Update();
}

void color_plane::Update(void)
{
	gui->DeleteDisplayLists(list_id, 1);
	
	xdir = GetLocData()->ydir.vpr(GetLocData()->zdir); i32s tmp1[2];
	for (tmp1[0] = 0;tmp1[0] < np;tmp1[0]++)
	{
		for (tmp1[1] = 0;tmp1[1] < np;tmp1[1]++)
		{
			fGL tmp2[3]; GetCRD(tmp1, tmp2); fGL tmp3[3];
			GetColor(GetValue(tmp2, ref, NULL) / value, tmp3);
			
			i32s index = tmp1[0] * np + tmp1[1];
			
			comp1[index] = tmp3[0];
			comp2[index] = tmp3[1];
			comp3[index] = tmp3[2];
		}
	}
}

// color_plane has a "convex" shape so it is not necessary to sort primitives when rendering
// a transparent plane... but we will run into problems if we have several such objects!!!

void color_plane::Render(void)
{
	if (glIsList(list_id) == GL_TRUE) glCallList(list_id);
	else
	{
		glNewList(list_id, GL_COMPILE_AND_EXECUTE);
		glPushMatrix(); SetModelView();
		
		const dummy_object * name = this;
		glPushName(GLNAME_OBJECT); glPushName((iGLu) name);
		
		fGL real_alpha = (transparent ? alpha : 1.0);
		
		glBegin(GL_QUADS);
		for (i32s n1 = 0;n1 < np - 1;n1++)
		{
			for (i32s n2 = 0;n2 < np - 1;n2++)
			{
				i32s tmp1, tmp2, tmp3;
				
				tmp1 = n1 + 0; tmp2 = n2 + 0; tmp3 = tmp1 * np + tmp2;
				glColor4f(comp1[tmp3], comp2[tmp3], comp3[tmp3], real_alpha);
				glVertex3f(dist[tmp1], dist[tmp2], 0.0);
				
				tmp1 = n1 + 1; tmp2 = n2 + 0; tmp3 = tmp1 * np + tmp2;
				glColor4f(comp1[tmp3], comp2[tmp3], comp3[tmp3], real_alpha);
				glVertex3f(dist[tmp1], dist[tmp2], 0.0);
				
				tmp1 = n1 + 1; tmp2 = n2 + 1; tmp3 = tmp1 * np + tmp2;
				glColor4f(comp1[tmp3], comp2[tmp3], comp3[tmp3], real_alpha);
				glVertex3f(dist[tmp1], dist[tmp2], 0.0);
				
				tmp1 = n1 + 0; tmp2 = n2 + 1; tmp3 = tmp1 * np + tmp2;
				glColor4f(comp1[tmp3], comp2[tmp3], comp3[tmp3], real_alpha);
				glVertex3f(dist[tmp1], dist[tmp2], 0.0);
			}
		}
		
		glEnd();
		
		fGL tmp1 = -0.55 * dim; fGL tmp2 = -0.45 * dim;
		
		glBegin(GL_LINES);
		glColor3f(1.0, 1.0, 1.0); glVertex3f(tmp1, tmp1, 0.0);
		glColor3f(0.0, 0.0, 1.0); glVertex3f(tmp2, tmp1, 0.0);
		
		glColor3f(1.0, 1.0, 1.0); glVertex3f(tmp1, tmp1, 0.0);
		glColor3f(0.0, 0.0, 1.0); glVertex3f(tmp1, tmp2, 0.0);
		
		glColor3f(1.0, 1.0, 1.0); glVertex3f(tmp1, tmp1, 0.0);
		glColor3f(0.0, 0.0, 1.0); glVertex3f(tmp1, tmp1, 0.1 * dim);
		glEnd();
		
		glPopName(); glPopName();
		glPopMatrix();
		
		glEndList();
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI

// eof
