// QM2GUI.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "qm2gui.h"

#ifdef ENABLE_GUI
#include "qm2eng1.h"
#include "qm2eng9.h"

#include "plane.h"
#include "surface.h"

#include "color.h"
#include "window.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

qm2_cm_element qm2_gui::cm_element = qm2_cm_element();

qm2_gui::qm2_gui(window_factory & p1, ostream * p2) : bs_gui(p1, p2), qm2_mdl(p2), bs_mdl_simple(p2)
{
}

qm2_gui::~qm2_gui(void)
{
}

// if we are going to trash the current engine, we must also trash all objects (planes/surfaces)
// that are dependent on it. and we must take care that the selected object remains valid...

void qm2_gui::DiscardCurrentEng(void)
{
	i32u n1 = 0;
	while (n1 < object_vector.size())
	{
		bool flag = false;
		
		color_plane * p_ref = dynamic_cast<color_plane *>(object_vector[n1]);
		if (p_ref != NULL && p_ref->ref == current_eng) flag = true;
		
		color_surface * s_ref = dynamic_cast<color_surface *>(object_vector[n1]);
		if (s_ref != NULL && s_ref->ref == current_eng) flag = true;
		
		if (flag)
		{
			if (selected_object == object_vector[n1]) selected_object = NULL;
			RemoveObject(object_vector[n1]);
		}
		else n1++;
	}
	
	qm2_mdl::DiscardCurrentEng();
}

const char * qm2_gui::GetType(void)
{
	return "QM, molecular calculations";
}

color_mode * qm2_gui::GetDefaultColorMode(void)
{
	return & qm2_gui::cm_element;
}

void qm2_gui::SelectAll(void)
{
}

void qm2_gui::InvertSelection(void)
{
}

// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!
// it seems that there is something common for all models (like the sel-buffer) -> move to the bs_gui?!?!

void qm2_gui::InitGL(void)
{
	glClearColor(0.0, 0.0, 0.0, 1.0);
	glDepthFunc(GL_LESS); glEnable(GL_DEPTH_TEST);
	
	glMateriali(GL_FRONT_AND_BACK, GL_SHININESS, 64);
	const fGL specular_reflectance[4] = { 0.9, 0.9, 0.9, 1.0 };
	glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular_reflectance);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);
	
	const fGL ambient_intensity[4] = { 0.2, 0.2, 0.2, 1.0 };
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient_intensity);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, true);
	
	glFogi(GL_FOG_MODE, GL_EXP); glFogf(GL_FOG_DENSITY, 0.15);
	const fGL fog_color[4] = { 0.0, 0.0, 0.0, 0.0 };
	glFogfv(GL_FOG_COLOR, fog_color);
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glSelectBuffer(SB_SIZE, select_buffer);
}

void qm2_gui::Render(view_window * win, rmode rm)
{
	bool accum = win->accumulate; if (rm != Normal) accum = false;
	if (accum) { glClear(GL_ACCUM_BUFFER_BIT); UpdateAccumValues(); }
	else if (rm != Transform2) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	if (win->enable_fog) glEnable(GL_FOG);
	
	//////////
	//////////
	
	for (i32u n1 = 0;n1 < cs_vector.size();n1++)
	{
		if (!cs_vector[n1].visible) continue;
		if (accum) glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
		if (win->render == RENDER_WIREFRAME)
		{
			glPointSize(3.0); glLineWidth(1.0);
			for (iter_qm2al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
//				if (!TestAtom(& (* it1), rm)) continue;
				glPushName(GLNAME_MD_TYPE1); glPushName((i32u) & (* it1));
				
				glBegin(GL_POINTS);
//				SetColor(win->colormode, & (* it1));
fGL_a3 color; win->colormode->GetColor(& (* it1), color); glColor3fv(color);

				glVertex3fv((* it1).crd_vector[n1].data);
				glEnd();
				
				glPopName(); glPopName();
			}
		}
		
		if (win->render != RENDER_WIREFRAME && win->render != RENDER_NOTHING)
		{
			glEnable(GL_LIGHTING);
			
			for (iter_qm2al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
//				if (!TestAtom(& (* it1), rm)) continue;
				
//				SetColor(win->colormode, & (* it1));
fGL_a3 color; win->colormode->GetColor(& (* it1), color); glColor3fv(color);
				
				float rad = (* it1).el.GetVDWRadius() * 0.125;
				int res = 12;
				
				glPushName(GLNAME_MD_TYPE1); glPushName((i32u) & (* it1));
				
				GLUquadricObj * qo = gluNewQuadric();
				gluQuadricDrawStyle(qo, (GLenum) GLU_FILL);
				glPushMatrix();
				glTranslated((* it1).crd_vector[n1][0], (* it1).crd_vector[n1][1], (* it1).crd_vector[n1][2]);
				gluSphere(qo, rad, res, res / 2);
				glPopMatrix();
				gluDeleteQuadric(qo);
				
				glPopName(); glPopName();
			}
			
			glDisable(GL_LIGHTING);
		}
		
		if (accum)
		{
			win->cam->RenderObjects(win);
			glAccum(GL_ACCUM, cs_vector[n1].accum_value);
		}
	}
	
	//////////
	//////////
	
	if (accum) glAccum(GL_RETURN, 1.0);
	else if (rm != Transform2) win->cam->RenderObjects(win);
	
/*	if (win->label == LABEL_ELEMENT)
	{
		glDisable(GL_DEPTH_TEST); glColor3f(1.0, 1.0, 1.0); char string[32];
		for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
		{
			glRasterPos3fv((* it1).crd_vector[0].data);
			
			ostrstream str(string, sizeof(string));
			str << (* it1).el.GetSymbol() << ends;
			win->WriteBitmapString(GLUT_BITMAP_9_BY_15, string);
		}
		glEnable(GL_DEPTH_TEST);  
	}*/
	
	if (win->enable_fog) glDisable(GL_FOG);
}

void qm2_gui::Center(transformer *)
{
}

void qm2_gui::Transform(transformer *)
{
}

// this comes directly from mm1_mdl, the bonds are just disabled...
// this comes directly from mm1_mdl, the bonds are just disabled...
// this comes directly from mm1_mdl, the bonds are just disabled...

void qm2_gui::DrawEvent(view_window * win, vector<iGLu> & names)
{
	if (window::button == mouse_tool::Right) return;	// the right button is for popup menus...
	
	i32s mouse[2] =
	{
		win->current_tool->latest_x,
		win->current_tool->latest_y
	};
	
	if (window::state == mouse_tool::Down)
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[0] = (qm2_atom *) names[1];
		}
		else
		{
			fGL tmp1[3]; win->GetCRD(mouse, tmp1);
			qm2_atom newatom(element::current_element, tmp1, cs_vector.size());
			AddAtom(newatom); draw_data[0] = & atom_list.back();
		}
	}
	else
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[1] = (qm2_atom *) names[1];
		}
		else
		{
			fGL tmp1[3]; win->GetCRD(mouse, tmp1);
			qm2_atom newatom(element::current_element, tmp1, cs_vector.size());
			AddAtom(newatom); draw_data[1] = & atom_list.back();
		}
		
		// if different: update bondtype or add a new bond.
		// if not different: change atom to different element.
		
		if (draw_data[0] != draw_data[1])
		{
//			mm1_bond newbond(draw_data[0], draw_data[1], bondtype::current_bondtype);
//			iter_mm1bl it1 = find(bond_list.begin(), bond_list.end(), newbond);
//			if (it1 != bond_list.end()) (* it1).bt = bondtype::current_bondtype;
//			else AddBond(newbond);
		}
		else
		{
			draw_data[0]->el = element::current_element;
		}
		
		UpdateAllViewWindows();
	}
}

// this comes directly from mm1_mdl, the bonds are just disabled...
// this comes directly from mm1_mdl, the bonds are just disabled...
// this comes directly from mm1_mdl, the bonds are just disabled...

void qm2_gui::EraseEvent(view_window * win, vector<iGLu> & names)
{
	if (window::button == mouse_tool::Right) return;	// the right button is for popup menus...
	
	if (window::state == mouse_tool::Down)
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[0] = (qm2_atom *) names[1];
		}
		else
		{
			draw_data[0] = NULL;
		}
	}
	else
	{
		if (names.size() > 1 && names[0] == GLNAME_MD_TYPE1)
		{
			draw_data[1] = (qm2_atom *) names[1];
		}
		else
		{
			draw_data[1] = NULL;
		}
		
		if (!draw_data[0] || !draw_data[1]) return;
		
		// if different: try to find and remove a bond.
		// if not different: remove atom.
		
		if (draw_data[0] != draw_data[1])
		{
//			mm1_bond tmpbond(draw_data[0], draw_data[1], bondtype::current_bondtype);
//			iter_mm1bl it1 = find(bond_list.begin(), bond_list.end(), tmpbond);
//			if (it1 != bond_list.end()) RemoveBond(it1); else return;
		}
		else
		{
			iter_qm2al it1 = find(atom_list.begin(), atom_list.end(), (* draw_data[0]));
			if (it1 != atom_list.end()) RemoveAtom(it1); else exit(EXIT_FAILURE);
		}
		
		UpdateAllViewWindows();
	}
}

void qm2_gui::SelectEvent(view_window *, vector<iGLu> &)
{
}

void qm2_gui::DoEnergy(void)
{

#ifdef ENABLE_MPQC

	if (!current_eng) current_eng = new qm2_eng_mpqc(* this);
	
	CopyCRD(this, current_eng, 0); current_eng->Compute(0);
	EnergyReport(current_eng);
	
	// we will not delete current_eng here, so that we can draw plots using it...
	// we will not delete current_eng here, so that we can draw plots using it...
	// we will not delete current_eng here, so that we can draw plots using it...
	
#endif	// ENABLE_MPQC

}

////////////////////////////////////////////////////////////////////////////////////////////////////

void qm2_cm_element::GetColor(const void * p1, fGL_a3 & p2)
{
	qm2_atom * ref = (qm2_atom *) p1;
	const fGL * color = ref->el.GetColor();
	for (i32s n1 = 0;n1 < 3;n1++) p2[n1] = color[n1];
}

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// ENABLE_GUI

// eof
