// T2MM1GUI.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "t2mm1gui.h"

#include "mm1alg.h"
#include "mm1eng1.h"
#include "mm1eng9.h"

#include "mm1rbn.h"

#include "plane.h"
#include "surface.h"

#include "color.h"
#include "window.h"

#include <fstream>
#include <strstream>

////////////////////////////////////////////////////////////////////////////////////////////////////

t2_mm1_gui::t2_mm1_gui(window_factory & p1, ostream * p2) : mm1_gui(p1, p2), bs_mdl_simple(p2)
{
}

t2_mm1_gui::~t2_mm1_gui(void)
{
}

static ifstream * traj = NULL;
static i32s nframes = NOT_DEFINED;
static i32s fcount = NOT_DEFINED;

void t2_mm1_gui::KeyEvent(view_window * win, char key, int, int)
{
	ifstream ifile;
	ofstream ofile;
	
	char input;
	
	mm1_eng * eng;
	
	char seq[1024];
	f64 aab_ah[] = { 302.0 * M_PI / 180.0, 313.0 * M_PI / 180.0, M_PI };
	f64 aab_bs[] = { 180.0 * M_PI / 180.0, 180.0 * M_PI / 180.0, M_PI };
	
	f64 nab[] =
	{
		44.6 * M_PI / 180.0, 261.0 * M_PI / 180.0, 320.8 * M_PI / 180.0, 208.6 * M_PI / 180.0,
		273.8 * M_PI / 180.0, 105.6 * M_PI / 180.0, 356.0 * M_PI / 180.0, 24.7 * M_PI / 180.0,
		88.7 * M_PI / 180.0, 264.6 * M_PI / 180.0
	};
	
	fGL tmp1; fGL tmp2;
	i32s sz[3] = { 50, 50, 50 };
	fGL dm[3] = { 0.55, 0.50, 0.45 };
	
	cp_param cpp;
	cpp.np = 30; cpp.gui = this; cpp.ref = this;
	
	cs_param csp;
	csp.np = sz; csp.dim = dm; csp.ostr = & cout;
	csp.gui = this; csp.ref = this;
	
	switch (key)
	{
		case 'h':
		cout << "+ -       -> change the current element" << endl;
		cout << "< >       -> change the current bondtype" << endl;
		cout << "b B       -> build sequences (amino/nucleic acids)" << endl;
		cout << "C         -> change color modes" << endl;
		cout << "r         -> change rendering modes (without updating view)" << endl;
		cout << "Q         -> change labels" << endl;
		cout << "i         -> import HIN/ENT files" << endl;
		cout << "O         -> open a file" << endl;
		cout << "S         -> save to a file" << endl;
		cout << "H         -> hydrogens, add or remove" << endl;
		cout << "e n m     -> make some nice planes/surfaces..." << endl;
		cout << "t         -> build a protein ribbon model" << endl;
		cout << "z         -> compare numerical and analytical gradient" << endl;
		cout << "E         -> compute energy [kJ/mol]" << endl;
		cout << "o         -> geometry optimization..." << endl;
		cout << "d         -> molecular dynamics..." << endl;
		cout << "X         -> trajectory mode on/off" << endl;
		cout << "1 2 7     -> (in traj-mode) get prev/next/desired frame" << endl;
		cout << "5         -> (in traj-mode) play the trajectory file" << endl;
		break;
		
		case '+': ++element::current_element;
		cout << "current element = " << element::current_element.GetAtomicNumber();
		cout << " " << element::current_element.GetSymbol() << endl;
		break;
		
		case '-': --element::current_element;
		cout << "current element = " << element::current_element.GetAtomicNumber();
		cout << " " << element::current_element.GetSymbol() << endl;
		break;
		
		case '<': ++bondtype::current_bondtype;
		cout << "current bondtype = " << bondtype::current_bondtype << endl;
		break;
		
		case '>': --bondtype::current_bondtype;
		cout << "current bondtype = " << bondtype::current_bondtype << endl;
		break;
		
		case 'b':
		cout << "give the sequence using symbols ARNDCQEGHILKMFPSTWYV ???"; cin >> seq;
		cout << "please select the conformation : H = helix, S = sheet ???"; cin >> input;
		mm1_mdl::amino_builder.Build(this, seq, input == 'h' || input == 'H' ? aab_ah : aab_bs);
		UpdateAllViewWindows();
		break;
		
		case 'B':
		cout << "give the sequence using symbols AGTC and aguc ???"; cin >> seq;
		mm1_mdl::nucleic_builder.Build(this, seq, nab);
		UpdateAllViewWindows();
		break;
		
		case 'C':
		if (win->colormode == & mm1_gui::cm_element) win->colormode = & mm1_gui::cm_state;
		else if (win->colormode == & mm1_gui::cm_state) win->colormode = & mm1_gui::cm_element;
		UpdateAllViewWindows();
		break;
		
		case 'r':
		cout << "rendering = ";
		switch (win->render)
		{
			case RENDER_NOTHING:
			win->render = RENDER_WIREFRAME;
			cout << "RENDER_WIREFRAME";
			break;
			
			case RENDER_WIREFRAME:
			win->render = RENDER_BALL_AND_STICK;
			cout << "RENDER_BALL_AND_STICK";
			break;
			
			case RENDER_BALL_AND_STICK:
			win->render = RENDER_VAN_DER_WAALS;
			cout << "RENDER_VAN_DER_WAALS";
			break;
			
			case RENDER_VAN_DER_WAALS:
			win->render = RENDER_CYLINDERS;
			cout << "RENDER_CYLINDERS";
			break;
			
			case RENDER_CYLINDERS:
			win->render = RENDER_NOTHING;
			cout << "RENDER_NOTHING";
			break;
		}
		cout << endl;
		break;
		
		case 'Q':
		cout << "label = ";
		switch (win->label)
		{
			case LABEL_NOTHING:
			win->label = LABEL_ELEMENT;
			cout << "LABEL_ELEMENT";
			break;
			
			case LABEL_ELEMENT:
			win->label = LABEL_BONDTYPE;
			cout << "LABEL_BONDTYPE";
			break;
			
			case LABEL_BONDTYPE:
			win->label = LABEL_NOTHING;
			cout << "LABEL_NOTHING";
			break;
		}
		cout << endl;
		break;
		
		case 'i':
		cout << "type of file to import (HIN or ENT) ? (h/e) ";
		cin >> input; if (input != 'h' && input != 'e') break;
		
		cout << "name of file to import ??? ";
		cin >> buffer;
		
		if (input == 'h') ImportHIN(buffer);
		if (input == 'e') ImportENT(buffer, NOT_DEFINED);
		
		UpdateAllViewWindows();
		break;
		
		case 'O':
		cout << "name of file to open ??? "; cin >> buffer;
		ifile.open(buffer, ios::in); ReadStream(ifile); ifile.close();
		UpdateAllViewWindows();
		break;
		
		case 'S':
		cout << "name of file to save ??? "; cin >> buffer;
		ofile.open(buffer, ios::out); WriteStream(ofile); ofile.close();
		break;
		
		case 'H':
		cout << "hydrogens, add or remove ? (a/r) ";
		cin >> input;
		
		if (input == 'a')
		{
			cout << "adding hydrogens..." << endl;
			AddHydrogens(); UpdateAllViewWindows();
		}
		
		if (input == 'r')
		{
			cout << "removing hydrogens..." << endl;
			RemoveHydrogens(); UpdateAllViewWindows();
		}
		break;
		
		case 'e':
		cout << "give dimension of the plane [nm] (1.0 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) mm1_GetESPValue; cpp.cf = GetRGBRange2;
		cpp.value = 4.1868 * 33.20716; cpp.descr = "ESP-";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case 'n': tmp1 = 4.1868 * 33.20716;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) mm1_GetESPValue; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetRedColor; csp.svalue = +tmp1; csp.descr = "pos ESP-";
		AddObject(new color_surface(ol_static(), csp));
		
		csp.cf = GetBlueColor; csp.svalue = -tmp1; csp.descr = "neg ESP-";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
		case 'm': tmp1 = 4.1868 * 33.20716 / 2.0;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false; csp.cf = GetRGBRange2;
		csp.vf1 = (ValueFunction *) mm1_GetSASValue; csp.vf2 = (ValueFunction *) mm1_GetESPValue;
		csp.svalue = 1.0; csp.cvalue = tmp1; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		csp.descr = "ESP-colored VDW-";
		
		AddObject(new color_surface(ol_static(), csp));
		UpdateAllViewWindows();
		break;
				
		case 't':
		mm1_mdl::amino_builder.Identify(this);
		DefineSecondaryStructure(this);
		
		AddObject(new mm1_ribbon(this, win->colormode, 0, 4));		// min. order is 2!!!
		UpdateAllViewWindows();
		break;
		
		case 'z':
		eng = new mm1_eng_exp9(* this, 1, 0);		// DEFAULT ENGINE!!!
//		eng = new mm1_eng_exp1(* this, 1, 0);
		CopyCRD(this, eng, 0); eng->Check(1);
		delete eng;
		break;
		
		case 'E':
		DoEnergy();
		break;
		
		case 'o':
		DoGeomOpt();
		break;
		
		case 'd':
		DoMolDyn();
		break;
		
		case 'X':
		if (traj != NULL)
		{
			cout << "traj mode off" << endl;
			traj->close(); delete traj; traj = NULL;
		}
		else
		{
			cout << "traj mode on" << endl;
			cout << "how many frames ??? "; cin >> nframes;
			cout << "traj-file name ??? "; cin >> buffer;
			traj = new ifstream(buffer, ios::in | ios::binary);
			fcount = 0;
		}
		break;
		
		case '1':
		case '2':
		case '7':
		if (traj != NULL)
		{
			switch (key)
			{
				case '1':
				fcount--; if (fcount < 0) fcount = 0;
				break;
				
				case '2':
				fcount++; if (fcount == nframes) fcount = nframes - 1;
				break;
				
				default:
				cout << "give the frame (in correct range) ??? "; cin >> fcount;
			}
			
			cout << "frame " << fcount << endl;
			
			i32s atoms = atom_list.size();
			i32s place = (2 + 3 * atoms) * sizeof(f64) * fcount;
			place += 2 * sizeof(f64);
			
			traj->seekg(place, ios::beg);
			
			for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
			{
				for (i32s t4 = 0;t4 < 3;t4++)
				{
					f64 t1a; traj->read((char *) & t1a, sizeof(t1a));
					(* it1).crd_vector[0][t4] = t1a;
				}
			}
			UpdateAllViewWindows();
		} else cout << "in traj-mode only!!!" << endl;
		break;
		
		case '5':
		if (traj != NULL)
		{
			i32s atoms = atom_list.size();
			for (i32s loop = 0;loop < nframes;loop++)
			{
				i32s place = (2 + 3 * atoms) * sizeof(f64) * loop;
				place += 2 * sizeof(f64);
				
				traj->seekg(place, ios::beg);
				
				for (iter_mm1al it1 = atom_list.begin();it1 != atom_list.end();it1++)
				{
					for (i32s t4 = 0;t4 < 3;t4++)
					{
						f64 t1a; traj->read((char *) & t1a, sizeof(t1a));
						(* it1).crd_vector[0][t4] = t1a;
					}
				}
				
				UpdateAllViewWindows(true);
			}
		} else cout << "in traj-mode only!!!" << endl;
		break;
		
		default:
		cout << "unknown command '" << key << "'" << endl;
	}
}

void t2_mm1_gui::EnergyReport(mm1_eng * eng)
{
	cout << "energy = " << eng->energy << " kJ/mol" << endl;
}

bool t2_mm1_gui::GeomOptGetParam(mm1_geomopt_param * param)
{
	cout << "how many cg-steps (500 might be Ok) ??? ";
	cin >> param->nsteps;
	
	return true;
}

#define START 10000	// see mm1gui.cpp!!!!!!!!

bool t2_mm1_gui::MolDynGetParam(mm1_moldyn_param * param)
{
	cout << "how many md-steps for simulation (10000 might be Ok) ??? ";
	cin >> param->nsteps;
	
	cout << "there are also " << START << " heating steps, total steps = " << (START + param->nsteps) << endl;
	cout << "the trajectory file will contain " << (param->nsteps / 100) << " frames!!!" << endl;
	
	cout << "time-step length [fs] (0.5 or less might be Ok) ??? ";
	cin >> param->timestep;
	
	cout << "traj-file name ??? ";
	cin >> param->filename;
	
	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
