// T2QM2GUI.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "t2qm2gui.h"

#include "qm2eng1.h"
#include "qm2eng9.h"

#include "plane.h"
#include "surface.h"

#include "color.h"
#include "window.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

t2_qm2_gui::t2_qm2_gui(window_factory & p1, ostream * p2) : qm2_gui(p1, p2), bs_mdl_simple(p2)
{
}

t2_qm2_gui::~t2_qm2_gui(void)
{
}

void t2_qm2_gui::KeyEvent(view_window * win, char key, int, int)
{
	fGL tmp1; fGL tmp2;
//	i32s sz[3] = { 50, 50, 50 };
i32s sz[3] = { 25, 25, 25 };					// make them quickly...
	fGL dm[3] = { 0.55, 0.50, 0.45 };
	
	cp_param cpp;
//	cpp.np = 30; cpp.gui = this; cpp.ref = current_eng;
cpp.np = 15; cpp.gui = this; cpp.ref = current_eng;		// make them quickly...

	cs_param csp;
	csp.np = sz; csp.dim = dm; csp.ostr = & cout;
	csp.gui = this; csp.ref = current_eng;
	
	char input;
	
	switch (key)
	{
		case 'h':
		cout << "+ -       -> change the current element" << endl;
		cout << "< >       -> change the current orbital" << endl;
		cout << "r         -> change rendering modes (without updating view)" << endl;
		cout << "i         -> import MM1-files" << endl;
		cout << "E         -> compute energy [Hartrees]" << endl;
//		cout << "f F       -> create electrostatic potential plane/surface" << endl;
		cout << "g G       -> create electron density plane/surface" << endl;
		cout << "n N       -> create molecular orbital plane/surface" << endl;
		cout << "m M       -> create MO-density plane/surface" << endl;
		break;
		
		case '+': ++element::current_element;
		cout << "current element = " << element::current_element.GetAtomicNumber();
		cout << " " << element::current_element.GetSymbol() << endl;
		break;
		
		case '-': --element::current_element;
		cout << "current element = " << element::current_element.GetAtomicNumber();
		cout << " " << element::current_element.GetSymbol() << endl;
		break;
		
		case '>': current_orbital++;
		cout << "current orbital = " << current_orbital << " ";
		cout << "(BE CAREFUL, THERE'S NO BOUNDS CHECKING YET!!!)" << endl;
		break;
		
		case '<': current_orbital--;
		cout << "current orbital = " << current_orbital << " ";
		cout << "(BE CAREFUL, THERE'S NO BOUNDS CHECKING YET!!!)" << endl;
		break;
		
		case 'r':
		cout << "rendering = ";
		switch (win->render)
		{
			case RENDER_NOTHING:
			win->render = RENDER_WIREFRAME;
			cout << "RENDER_WIREFRAME";
			break;
			
			case RENDER_WIREFRAME:
			win->render = RENDER_BALL_AND_STICK;
			cout << "RENDER_BALL_AND_STICK";
			break;
			
			case RENDER_BALL_AND_STICK:
			win->render = RENDER_NOTHING;
			cout << "RENDER_NOTHING";
			break;
		}
		cout << endl;
		break;
		
		case 'i':
		cout << "name of file to import ??? ";
		cin >> buffer;
		
		ImportMM1(buffer);
		UpdateAllViewWindows();
		break;
		
// from now on we are 100% dependent on MPQC...
// from now on we are 100% dependent on MPQC...
// from now on we are 100% dependent on MPQC...

#ifdef ENABLE_MPQC

		case 'E':
		DoEnergy();
		break;
		
// this "cpp.ref"-thing is not yet finished. won't work with other engines than mpqc...
// this "cpp.ref"-thing is not yet finished. won't work with other engines than mpqc...
// this "cpp.ref"-thing is not yet finished. won't work with other engines than mpqc...

		case 'g':
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) qm2_eng_mpqc::getElDens; cpp.cf = GetGreenColor;
		cpp.value = 0.05; cpp.descr = "E-density ";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case 'G': tmp1 = 0.05;
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) qm2_eng_mpqc::getElDens; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetGreenColor; csp.svalue = +tmp1; csp.descr = "E-density ";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
		case 'n':
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) qm2_eng_mpqc::getOrbital; cpp.cf = GetRGBRange2;
		cpp.value = 0.05; cpp.descr = "molecular orbital ";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case 'N': tmp1 = 0.05;
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) qm2_eng_mpqc::getOrbital; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetRedColor; csp.svalue = +tmp1; csp.descr = "pos MO ";
		AddObject(new color_surface(ol_static(), csp));
		
		csp.cf = GetBlueColor; csp.svalue = -tmp1; csp.descr = "neg MO ";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
		case 'm':
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) qm2_eng_mpqc::getOrbDens; cpp.cf = GetGreenColor;
		cpp.value = 0.0125; cpp.descr = "MO-density ";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case 'M': tmp1 = 0.0125;
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) qm2_eng_mpqc::getOrbDens; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetGreenColor; csp.svalue = +tmp1; csp.descr = "MO-density ";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
#endif	// ENABLE_MPQC

	}
}

void t2_qm2_gui::EnergyReport(qm2_eng * eng)
{
	cout << "energy = " << eng->energy << " Hartrees" << endl;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
