// T2WINDOW.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "t2window.h"

#include "guibase.h"
#include "utility.h"

#include <algorithm>

////////////////////////////////////////////////////////////////////////////////////////////////////

vector<glut_window *> glut_window::glut_vector;

glut_window::glut_window(bs_gui * p1) : window(p1)
{
	glut_id = NOT_DEFINED;
}

glut_window::~glut_window(void)
{
	vector<glut_window *>::iterator it1;
	it1 = find(glut_vector.begin(), glut_vector.end(), this);
	
	if (it1 != glut_vector.end())
	{
		glut_vector.erase(it1);
		glutDestroyWindow(glut_id);
	}
}

void glut_window::CreateWindow(void)
{
	glut_vector.push_back(this);
	
	glut_id = glutCreateWindow(""); gui->InitGL();
	
	glutDisplayFunc(glut_window::DisplayHandler);
	glutKeyboardFunc(glut_window::KeyboardHandler);
	glutSpecialFunc(glut_window::SpecialHandler);
	
	glutReshapeFunc(glut_window::ReshapeHandler);
	
	glutMouseFunc(glut_window::MouseHandler);
	glutMotionFunc(glut_window::MotionHandler);
}

void glut_window::SetTitle(const char * p1)
{
	SetCurrent();
	glutSetWindowTitle(p1);
}

void glut_window::SetCurrent(void)
{
	glutSetWindow(glut_id);
}

void glut_window::Update(bool directly)
{
	if (directly) DisplayEvent();
	else
	{
		SetCurrent();
		glutPostRedisplay();
	}
}

glut_window * glut_window::GetWindow(int id)
{
	vector<glut_window *>::iterator it1 = glut_vector.begin();
	while (it1 != glut_vector.end())
	{
		vector<glut_window *>::iterator it2 = it1++;
		if ((* it2)->glut_id == id) return (* it2);
	}
	
	// return NULL if the search failed...
	// return NULL if the search failed...
	// return NULL if the search failed...
	
	return NULL;
}

void glut_window::DisplayHandler(void)
{
	glut_window * win = GetWindow(glutGetWindow());
	if (!win) cout << "Unknown ID in DisplayHandler !!!" << endl;
	else win->DisplayEvent();
}

void glut_window::KeyboardHandler(unsigned char key, int x, int y)
{
	glut_window * win = GetWindow(glutGetWindow());
	if (!win) cout << "Unknown ID in KeyboardHandler !!!" << endl;
	else
	{
		// handle "remove window"-commands here to prevent "delete this"-situation!!!
		// handle "remove window"-commands here to prevent "delete this"-situation!!!
		// handle "remove window"-commands here to prevent "delete this"-situation!!!
		
		if (key == 'V')
		{
			view_window * view_win = dynamic_cast<view_window *>(win);
			if (view_win != NULL) view_win->gui->RemoveViewWindow(view_win);
		}
		else win->KeyboardEvent(key, x, y);
	}
}

void glut_window::SpecialHandler(int key, int x, int y)
{
	glut_window * win = GetWindow(glutGetWindow());
	if (!win) cout << "Unknown ID in SpecialHandler !!!" << endl;
	else win->SpecialEvent(key, x, y);
}

void glut_window::ReshapeHandler(int width, int height)
{
	glut_window * win = GetWindow(glutGetWindow());
	if (!win) cout << "Unknown ID in ReshapeHandler !!!" << endl;
	else win->SetSize(width, height);
}

void glut_window::MouseHandler(int ibtn, int istt, int x, int y)
{
	glut_window * win = GetWindow(glutGetWindow());
	if (!win) cout << "Unknown ID in MouseHandler !!!" << endl;
	else
	{
		view_window * view_win = dynamic_cast<view_window *>(win);
		
		if (istt == GLUT_DOWN)
		{
			if (button == mouse_tool::None)
			{
				switch (ibtn)
				{
					case GLUT_LEFT_BUTTON:
					button = mouse_tool::Left;
					break;
					
					case GLUT_RIGHT_BUTTON:
					button = mouse_tool::Right;
					break;
					
					default:
					button = mouse_tool::Middle;
				}
				
				int mod = glutGetModifiers();
				shift_down = (mod & GLUT_ACTIVE_SHIFT) ? true : false;
				ctrl_down = (mod & GLUT_ACTIVE_CTRL) ? true : false;
				
				state = mouse_tool::Down;
				
				if (view_win != NULL) current_tool->ButtonEvent(view_win, x, y);
			}
		}
		else
		{
			if (button == mouse_tool::Left && ibtn != GLUT_LEFT_BUTTON) return;
			if (button == mouse_tool::Middle && ibtn != GLUT_MIDDLE_BUTTON) return;
			if (button == mouse_tool::Right && ibtn != GLUT_RIGHT_BUTTON) return;
			
			state = mouse_tool::Up;
			
			if (view_win != NULL) current_tool->ButtonEvent(view_win, x, y);
			
			button = mouse_tool::None;
		}
	}
}

void glut_window::MotionHandler(int x, int y)
{
	glut_window * win = GetWindow(glutGetWindow());
	if (!win) cout << "Unknown ID in MotionHandler !!!" << endl;
	else
	{
		view_window * view_win = dynamic_cast<view_window *>(win);
		if (view_win != NULL) current_tool->MotionEvent(view_win, x, y);
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

glut_view_window::glut_view_window(bs_gui * p1, camera * p2) : glut_window(p1), view_window(p1, p2), window(p1)
{
}

glut_view_window::~glut_view_window(void)
{
}

void glut_view_window::DisplayEvent(void)
{
	SetCurrent();
	Render(); glFlush();
	glutSwapBuffers();
}

void glut_view_window::KeyboardEvent(unsigned char key, int x, int y)
{
	char input;
	light * new_light;
	
	switch (key)
	{
		case 'h':
		cout << endl;
		cout << "modifier keys:" << endl;
		cout << "--------------" << endl;
		cout << "SHIFT     -> manipulate the selected object/atoms" << endl;
		cout << "CTRL      -> use object's own orientation (for translations only)" << endl;
		
		cout << endl;
		cout << "general keyboard commands:" << endl;
		cout << "--------------------------" << endl;
		cout << "F1-F10    -> change the current tool" << endl;
		cout << "F11 F12   -> select all / invert selection" << endl;
		cout << "HOME      -> switch the fog effect on/off" << endl;
		cout << "END       -> switch accumulation on/off" << endl;
		cout << "h         -> print out all available keyboard commands" << endl;
		cout << "P         -> switch orhographic/perspective projections" << endl;
		cout << "U         -> switch the quick update mode on/off" << endl;
		cout << "I         -> switch the info line on/off" << endl;
		cout << "c         -> create a new camera with a new view" << endl;
		cout << "v         -> create a new view using the current view camera" << endl;
		cout << "V         -> remove the current view" << endl;
		cout << "u         -> update the current view" << endl;
		cout << "A         -> add a new light" << endl;
		cout << "k K       -> switch local or global lights on/off" << endl;
		cout << "l L       -> select next local or global light" << endl;
		cout << "D         -> delete the currently selected object" << endl;
		
		cout << endl;
		cout << "model-dependent keyboard commands:" << endl;
		cout << "----------------------------------" << endl;
		gui->KeyEvent(this, key, x, y); cout << endl;
		break;
		
		case 'P' :
		cam->ortho = !cam->ortho; gui->UpdateViewWindows(cam);
		cout << "projection = " << (cam->ortho ? "orthographic" : "perspective") << endl;
		break;
		
		case 'U':
		quick_update = !quick_update;
		cout << "quick update = " << (quick_update ? "on" : "off") << endl;
		break;
		
		case 'I':
		draw_info = !draw_info;
		cout << "info line = " << (draw_info ? "on" : "off") << endl;
		gui->UpdateAllViewWindows();
		break;
		
		case 'c':
		gui->AddCamera(cam->focus);
		break;
		
		case 'v':
		gui->AddViewWindow(cam);
		break;
		
		// case 'V' is handled in glut_window::KeyboardHandler()!!!!!!!!!
		// case 'V' is handled in glut_window::KeyboardHandler()!!!!!!!!!
		// case 'V' is handled in glut_window::KeyboardHandler()!!!!!!!!!
		
		case 'u':
		cout << "update" << endl;
		gui->UpdateViewWindow(this);
		break;
		
		case 'A':
		cout << "add light:" << endl;
		cout << "directional- or spotlight ? (d/s) "; cin >> input;
		
		if (input == 'd') new_light = new directional_light(ol_static());
		else if (input == 's')
		{
			new_light = new spot_light(ol_static());
			
			const fGL trans[3] = { 0.0, 0.0, -1.0 };
			new_light->TranslateObject((const fGL *) trans, cam->GetLocData());
		}
		else break;
		
		cout << "global or local light ? (g/l) "; cin >> input;
		
		if (input == 'g') gui->AddGlobalLight(new_light);
		else if (input == 'l') gui->AddLocalLight(new_light, cam);
		else delete new_light;
		
		if (input == 'g' || draw_info) gui->UpdateAllViewWindows();
		else if (input == 'l') gui->UpdateViewWindows(cam);
		break;
		
		case 'k':
		gui->DoSwitchLocalLights(cam, true);
		break;
		
		case 'K':
		gui->DoSwitchGlobalLights(cam, true);
		break;
		
		case 'l':
		if (gui->CountLocalLights(cam))
		{
			i32s tmp1 = (gui->IsLight(gui->selected_object) + 1) % gui->light_vector.size();
			while (gui->light_vector[tmp1]->owner != cam) tmp1 = (tmp1 + 1) % gui->light_vector.size();
			gui->SelectLight(gui->light_vector[tmp1]); if (draw_info) gui->UpdateAllViewWindows();
		}
		else cout << "no local lights to select" << endl;
		break;
		
		case 'L':
		if (gui->CountGlobalLights())
		{
			i32s tmp1 = (gui->IsLight(gui->selected_object) + 1) % gui->light_vector.size();
			while (gui->light_vector[tmp1]->owner != NULL) tmp1 = (tmp1 + 1) % gui->light_vector.size();
			gui->SelectLight(gui->light_vector[tmp1]); if (draw_info) gui->UpdateAllViewWindows();
		}
		else cout << "no global lights to select" << endl;
		break;
		
		case 'D':
		gui->DoDeleteCurrentObject();
		break;
		
		default:
		gui->KeyEvent(this, key, x, y);
		break;
	}
}

void glut_view_window::SpecialEvent(int key, int, int)
{
	switch (key)
	{
		case GLUT_KEY_F1: case GLUT_KEY_F2: case GLUT_KEY_F3: case GLUT_KEY_F4: case GLUT_KEY_F5:
		case GLUT_KEY_F6: case GLUT_KEY_F7: case GLUT_KEY_F8: case GLUT_KEY_F9: case GLUT_KEY_F10:
		if (window::button == mouse_tool::None) break;
		
		cout << "tool in use, cannot change at the moment..." << endl;
		return;
	}
	
	switch (key)
	{
		case GLUT_KEY_F1:
		current_tool = & window::tool_draw;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F2:
		current_tool = & window::tool_erase;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F3:
		current_tool = & window::tool_select;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F4:
		current_tool = & window::tool_zoom;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F5:
		current_tool = & window::tool_translate_xy;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F6:
		current_tool = & window::tool_translate_z;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F7:
		current_tool = & window::tool_orbit_xy;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F8:
		current_tool = & window::tool_orbit_z;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F9:
		current_tool = & window::tool_rotate_xy;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_F10:
		current_tool = & window::tool_rotate_z;
		cout << "tool = " << current_tool->GetToolName() << endl;
		if (draw_info) gui->UpdateAllViewWindows();
		break;
		
		case GLUT_KEY_HOME: enable_fog = !enable_fog; Update(false);
		cout << "fog effect = " << (enable_fog ? "TRUE" : "FALSE") << endl;
		break;
		
		case GLUT_KEY_END: accumulate = !accumulate; Update(false);
		cout << "accumulation = " << (accumulate ? "TRUE" : "FALSE") << endl;
//		if (accumulate) gui->UpdateAccumValues();	// ?????
		break;
		
		case GLUT_KEY_F11: cout << "select all" << endl;
		gui->SelectAll(); gui->UpdateAllViewWindows(false);
		break;
		
		case GLUT_KEY_F12: cout << "invert selection" << endl;
		gui->InvertSelection(); gui->UpdateAllViewWindows(false);
		break;
		
		default:
		cout << "glut_window::SpecialEvent(" << key << ")" << endl;
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////////////////////////

glut_factory * glut_factory::instance = NULL;
singleton_cleaner<glut_factory> glut_factory_cleaner(glut_factory::GetInstance());

glut_factory::glut_factory() : window_factory()
{
}

glut_factory::~glut_factory()
{
}

glut_factory * glut_factory::GetInstance(void)
{
	if (instance != NULL) return instance;
	else return (instance = new glut_factory());
}

view_window * glut_factory::ProduceView(bs_gui * mdl, camera * cam)
{
	glut_view_window * win = new glut_view_window(mdl, cam);
	win->CreateWindow(); return win;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
