// T2WINDOW.H : a view window class for GLUT.

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is for TARGET2...

#ifndef T2WINDOW_H
#define T2WINDOW_H

class glut_window;
class glut_view_window;

class glut_factory;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "window.h"

#include <vector>

////////////////////////////////////////////////////////////////////////////////////////////////////

class glut_window : virtual public window
{
	private:
	
	int glut_id;
	
	private:
	
	static vector<glut_window *> glut_vector;
	
	public:
	
	glut_window(bs_gui *);
	virtual ~glut_window(void);
	
	void CreateWindow(void);		// virtual
	void SetTitle(const char *);		// virtual
	void SetCurrent(void);			// virtual
	void Update(bool);			// virtual
	
	private:
	
	virtual void DisplayEvent(void) = 0;
	virtual void KeyboardEvent(unsigned char, int, int) = 0;
	virtual void SpecialEvent(int, int, int) = 0;
	
	private:
	
	static glut_window * GetWindow(int);
	
	static void DisplayHandler(void);
	static void KeyboardHandler(unsigned char, int, int);
	static void SpecialHandler(int, int, int);
	
	static void ReshapeHandler(int, int);
	
	static void MouseHandler(int, int, int, int);
	static void MotionHandler(int, int);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class glut_view_window : public glut_window, public view_window
{
	public:
	
	glut_view_window(bs_gui *, camera *);
	~glut_view_window(void);
	
	private:
	
	void DisplayEvent(void);				// virtual
	void KeyboardEvent(unsigned char, int, int);		// virtual
	void SpecialEvent(int, int, int);			// virtual
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class glut_factory : public window_factory
{
	private:
	
	static glut_factory * instance;
	glut_factory(void);
	
	public:
	
	~glut_factory(void);
	static glut_factory * GetInstance(void);
	
	view_window * ProduceView(bs_gui *, camera *);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif	// T2WINDOW_H

// eof
