// T3DIALOG.CPP

// Copyright (C) 1999 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "t3dialog.h"

#include "utility.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

GtkWidget * element_dialog::window;

element_dialog::element_dialog(void)
{
	window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(window), "Set Current Element");
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
	
	GtkWidget * table = gtk_table_new(21, 38, TRUE);
	gtk_container_add(GTK_CONTAINER(window), table);
	
	int attachdata[ELEMENT_SYMBOLS][2] =
	{
		{ 1, 1 }, { 35, 1 }, { 1, 3 }, { 3, 3 }, { 25, 3 },		// B
		{ 27, 3 }, { 29, 3 }, { 31, 3 }, { 33, 3 }, { 35, 3 },		// Ne
		{ 1, 5 }, { 3, 5 }, { 25, 5 }, { 27, 5 }, { 29, 5 },		// P
		{ 31, 5 }, { 33, 5 }, { 35, 5 }, { 1, 7 }, { 3, 7 },		// Ca
		{ 5, 7 }, { 7, 7 }, { 9, 7 }, { 11, 7 }, { 13, 7 },		// Mn
		{ 15, 7 }, { 17, 7 }, { 19, 7 }, { 21, 7 }, { 23, 7 },		// Zn
		{ 25, 7 }, { 27, 7 }, { 29, 7 }, { 31, 7 }, { 33, 7 },		// Br
		{ 35, 7 }, { 1, 9 }, { 3, 9 }, { 5, 9 }, { 7, 9 },		// Zr
		{ 9, 9 }, { 11, 9 }, { 13, 9 }, { 15, 9 }, { 17, 9 },		// Rh
		{ 19, 9 }, { 21, 9 }, { 23, 9 }, { 25, 9 }, { 27, 9 },		// Sn
		{ 29, 9 }, { 31, 9 }, { 33, 9 }, { 35, 9 }, { 1, 11 },		// Cs
		{ 3, 11 }, { 5, 11 }, { 9, 16 }, { 11, 16 }, { 13, 16 },	// Nd
		{ 15, 16 }, { 17, 16 }, { 19, 16 }, { 21, 16 }, { 23, 16 },	// Tb
		{ 25, 16 }, { 27, 16 }, { 29, 16 }, { 31, 16 }, { 33, 16 },	// Yb
		{ 35, 16 }, { 7, 11 }, { 9, 11 }, { 11, 11 }, { 13, 11 },	// Re
		{ 15, 11 }, { 17, 11 }, { 19, 11 }, { 21, 11 }, { 23, 11 },	// Hg
		{ 25, 11 }, { 27, 11 }, { 29, 11 }, { 31, 11 }, { 33, 11 },	// At
		{ 35, 11 }, { 1, 13 }, { 3, 13 }, { 5, 13 }, { 9, 18 },		// Th
		{ 11, 18 }, { 13, 18 }, { 15, 18 }, { 17, 18 }, { 19, 18 },	// Am
		{ 21, 18 }, { 23, 18 }, { 25, 18 }, { 27, 18 }, { 29, 18 },	// Fm
		{ 31, 18 }, { 33, 18 }, { 35, 18 }, { 7, 13 }, { 9, 13 }	// Ha
	};
	
	GtkSignalFunc sf = (GtkSignalFunc) element_dialog::SignalHandler; GtkWidget * button;
	
	for (int n1 = 0;n1 < ELEMENT_SYMBOLS;n1++)
	{
		element tmp_element(n1 + 1);
		button = gtk_button_new_with_label(tmp_element.GetSymbol());
		gtk_signal_connect(GTK_OBJECT(button), "clicked", sf, (void *) (n1 + 1));
		gtk_widget_show(button);
		
		int x = attachdata[n1][0]; int y = attachdata[n1][1];
		gtk_table_attach_defaults(GTK_TABLE(table), button, x, x + 2, y, y + 2);
	}
	
	gtk_widget_show(table);
	gtk_widget_show(window);
}

element_dialog::~element_dialog(void)
{
}

void element_dialog::SignalHandler(GtkWidget *, int signal)
{
	element::current_element = element(signal);
	cout << "current element is now " << element::current_element.GetSymbol() << endl;
	
	gtk_widget_destroy(window);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

GtkWidget * bondtype_dialog::window;

bondtype_dialog::bondtype_dialog(void)
{
	window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_window_set_title(GTK_WINDOW(window), "Set Current Bondtype");
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
	
	GtkWidget * table = gtk_table_new((2 + 4), 5, TRUE);
	gtk_container_add(GTK_CONTAINER(window), table);
	
	char bt_symbols[4] = { 'S', 'C', 'D', 'T' };
	
	GtkSignalFunc sf = (GtkSignalFunc) bondtype_dialog::SignalHandler; GtkWidget * button;
	
	for (int n1 = 0;n1 < 4;n1++)
	{
		bondtype tmp_bondtype(bt_symbols[n1]);
		button = gtk_button_new_with_label(tmp_bondtype.GetString());
		gtk_signal_connect(GTK_OBJECT(button), "clicked", sf, (void *) bt_symbols[n1]);
		gtk_widget_show(button);
		
		int x = 2; int y = n1 + 1;
		gtk_table_attach_defaults(GTK_TABLE(table), button, x, x + 1, y, y + 1);
	}
	
	gtk_widget_show(table);
	gtk_widget_show(window);
}

bondtype_dialog::~bondtype_dialog(void)
{
}

void bondtype_dialog::SignalHandler(GtkWidget *, int signal)
{
	bondtype::current_bondtype = bondtype((char) signal);
	cout << "current bondtype is now " << bondtype::current_bondtype.GetString() << endl;
	
	gtk_widget_destroy(window);
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
