// T3MM1GUI.CPP

// Copyright (C) 1998 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "t3mm1gui.h"

#include "t3dialog.h"
#include "t3window.h"

#include "mm1alg.h"
#include "mm1eng1.h"
#include "mm1eng9.h"

#include "mm1rbn.h"

#include "plane.h"
#include "surface.h"

#include "color.h"
//#include "window.h"

#include <fstream>
#include <strstream>

////////////////////////////////////////////////////////////////////////////////////////////////////

GnomeUIInfo t3_mm1_gui::fileimport_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Protein Databank PDB", "pdb",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_IMPORT_PDB, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "HyperChem HIN", "hin",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_IMPORT_HIN, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::file_submenu[] =
{
	GNOMEUIINFO_MENU_OPEN_ITEM(gtkgl_window::GNOMEPopupHandler, SIGNAL_MM1_FILE_OPEN),
	GNOMEUIINFO_SUBTREE("_Import", fileimport_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_SAVE_AS_ITEM(gtkgl_window::GNOMEPopupHandler, SIGNAL_MM1_FILE_SAVEAS),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_CLOSE_ITEM(gtkgl_window::GNOMEPopupHandler, SIGNAL_MM1_FILE_CLOSE),
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::select_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Select All", "select all atoms...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_SELECT_SELECTALL, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Invert Selection", "invert the selection...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_SELECT_INVERTSELECTION, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::settings_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Set Current Element", "el...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_SETTINGS_ELEMENT, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Set Current Bondtype", "bt...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_SETTINGS_BONDTYPE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::viewscreate_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "New Camera", "create a new view with a new camera...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_VIEWS_NEWCAMERA, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "New View", "create a new view using the current camera...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_VIEWS_NEWVIEW, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::views_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_Create", viewscreate_submenu),
	
	{
		GNOME_APP_UI_ITEM, "Delete This View", "remove this view...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_VIEWS_DELETEVIEW, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::lights_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Create New Light", "create a new light object...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_LIGHTS_NEWLIGHT, NULL
	},
	
	GNOMEUIINFO_SEPARATOR,
	
	{
		GNOME_APP_UI_ITEM, "Switch Local Lights", "loc",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_LIGHTS_SWITCHLOCAL, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Switch Global Lights", "glob",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_LIGHTS_SWITCHGLOBAL, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::rendermode_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Wireframe", "wf",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_RMODE_WIREFRAME, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Ball & Stick", "bs",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_RMODE_BALLANDSTICK, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "vanderWaals", "vdw",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_RMODE_VANDERWAALS, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Cylinders", "cyl",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_RMODE_CYLINDERS, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Nothing", "null",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_RMODE_NOTHING, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::projection_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Orthographic", "ortho",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_PROJECTION_ORTHOGRAPHIC, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Perspective", "persp",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_PROJECTION_PERSPECTIVE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::render_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_Views", views_submenu),
	GNOMEUIINFO_SUBTREE("_Lights", lights_submenu),
	GNOMEUIINFO_SUBTREE("_Rendering Mode", rendermode_submenu),
	GNOMEUIINFO_SUBTREE("_Projection", projection_submenu),
	
	{
		GNOME_APP_UI_ITEM, "Quick Update Mode", "quick update...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_RENDER_QUICKUPDATE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::objects_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Ribbon", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_OBJECTS_RIBBON, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "ESP-plane", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_OBJECTS_EPLANE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "ESP-surface", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_OBJECTS_ESURFACE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "ESP-colored vdW-surface", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_OBJECTS_VDWSURFACE, NULL
	},
	
	GNOMEUIINFO_SEPARATOR,
	
	{
		GNOME_APP_UI_ITEM, "Delete Current Object", "delete",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_OBJECTS_DELCURROBJ, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::compute_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Energy", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_COMPUTE_ENERGY, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Geometry Optimization...", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_COMPUTE_GEOMOPT, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Molecular Dynamics...", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_COMPUTE_MOLDYN, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_mm1_gui::popupmenu_info[] =
{
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	
	GNOMEUIINFO_SUBTREE("_File", file_submenu),
	GNOMEUIINFO_SUBTREE("_Select", select_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Settings...", settings_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Render", render_submenu),
	GNOMEUIINFO_SUBTREE("_Objects", objects_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Compute", compute_submenu),
	GNOMEUIINFO_SEPARATOR,
	
	{
		GNOME_APP_UI_ITEM, "Hydrogens...", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_HYDROGENS, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Sequence Builder...", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_MM1_BUILDER, NULL
	},
	
	GNOMEUIINFO_END
};

////////////////////////////////////////////////////////////////////////////////////////////////////

t3_mm1_gui::t3_mm1_gui(window_factory & p1, ostream * p2) : mm1_gui(p1, p2), bs_mdl_simple(p2)
{
	popupmenu = gnome_popup_menu_new(popupmenu_info);
}

t3_mm1_gui::~t3_mm1_gui(void)
{
}

void t3_mm1_gui::PopupEvent(view_window * win, int signal)
{
	char input;
	ifstream ifile;
	ofstream ofile;
	
	char seq[1024];
	f64 aab_ah[] = { 302.0 * M_PI / 180.0, 313.0 * M_PI / 180.0, M_PI };
	f64 aab_bs[] = { 180.0 * M_PI / 180.0, 180.0 * M_PI / 180.0, M_PI };
	
	f64 nab[] =
	{
		44.6 * M_PI / 180.0, 261.0 * M_PI / 180.0, 320.8 * M_PI / 180.0, 208.6 * M_PI / 180.0,
		273.8 * M_PI / 180.0, 105.6 * M_PI / 180.0, 356.0 * M_PI / 180.0, 24.7 * M_PI / 180.0,
		88.7 * M_PI / 180.0, 264.6 * M_PI / 180.0
	};
	
	fGL tmp1; fGL tmp2;
	i32s sz[3] = { 50, 50, 50 };
	fGL dm[3] = { 0.55, 0.50, 0.45 };
	
	cp_param cpp;
	cpp.np = 30; cpp.gui = this; cpp.ref = this;
	
	cs_param csp;
	csp.np = sz; csp.dim = dm; csp.ostr = & cout;
	csp.gui = this; csp.ref = this;
	
	switch (signal)
	{
		case SIGNAL_MM1_FILE_OPEN:
		cout << "name of file to open ??? "; cin >> buffer;
		ifile.open(buffer, ios::in); ReadStream(ifile); ifile.close();
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_IMPORT_PDB:
		cout << "name of PDB file to import ??? ";
		cin >> buffer; ImportENT(buffer, NOT_DEFINED);
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_IMPORT_HIN:
		cout << "name of HIN file to import ??? ";
		cin >> buffer; ImportHIN(buffer);
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_FILE_SAVEAS:
		cout << "name of file to save ??? "; cin >> buffer;
		ofile.open(buffer, ios::out); WriteStream(ofile); ofile.close();
		break;
		
		case SIGNAL_MM1_FILE_CLOSE:
		cout << "this should close all windows and delete the model." << endl;
		cout << "however, it's not yet implemented :) Sorry!!!" << endl;
		break;
		
		case SIGNAL_MM1_SETTINGS_ELEMENT:
		new element_dialog();			// memory leak???
		break;
		
		case SIGNAL_MM1_SETTINGS_BONDTYPE:
		new bondtype_dialog();			// memory leak???
		break;
		
		case SIGNAL_MM1_HYDROGENS:
		cout << "hydrogens, add or remove ? (a/r) "; cin >> input;
		
		if (input == 'a')
		{
			cout << "adding hydrogens..." << endl;
			AddHydrogens(); UpdateAllViewWindows();
		}
		
		if (input == 'r')
		{
			cout << "removing hydrogens..." << endl;
			RemoveHydrogens(); UpdateAllViewWindows();
		}
		break;
		
		case SIGNAL_MM1_BUILDER:
		cout << "build amino/nucleic acid sequences ? (a/n) "; cin >> input;
		
		if (input == 'a')
		{
			cout << "give the sequence using symbols ARNDCQEGHILKMFPSTWYV ???"; cin >> seq;
			cout << "please select the conformation : H = helix, S = sheet ???"; cin >> input;
			mm1_mdl::amino_builder.Build(this, seq, input == 'h' || input == 'H' ? aab_ah : aab_bs);
			UpdateAllViewWindows();
		}
		
		if (input == 'n')
		{
			cout << "give the sequence using symbols AGTC and aguc ???"; cin >> seq;
			mm1_mdl::nucleic_builder.Build(this, seq, nab);
			UpdateAllViewWindows();
		}
		break;
		
/*		case SIGNAL_POPUP_MM1_RENDER_CMODE_STATE:
		win->colormode = & t3_mm1_gui::cm_state; UpdateAllViewWindows();
		break;
		
		case SIGNAL_POPUP_MM1_RENDER_CMODE_ELEMENT:
		win->colormode = & t3_mm1_gui::cm_element; UpdateAllViewWindows();
		break;		*/
		
		case SIGNAL_MM1_RMODE_WIREFRAME:
		win->render = RENDER_WIREFRAME; UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_RMODE_BALLANDSTICK:
		win->render = RENDER_BALL_AND_STICK; UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_RMODE_VANDERWAALS:
		win->render = RENDER_VAN_DER_WAALS; UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_RMODE_CYLINDERS:
		win->render = RENDER_CYLINDERS; UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_RMODE_NOTHING:
		win->render = RENDER_NOTHING; UpdateAllViewWindows();
		break;
		
/*		case SIGNAL_POPUP_MM1_RENDER_LABEL_ELEMENT:
		win->label = LABEL_ELEMENT; UpdateAllViewWindows();
		break;
		
		case SIGNAL_POPUP_MM1_RENDER_LABEL_BONDTYPE:
		win->label = LABEL_BONDTYPE; UpdateAllViewWindows();
		break;
		
		case SIGNAL_POPUP_MM1_RENDER_LABEL_NOTHING:
		win->label = LABEL_NOTHING; UpdateAllViewWindows();
		break;		*/
		
		case SIGNAL_MM1_COMPUTE_ENERGY:
		DoEnergy();
		break;
		
		case SIGNAL_MM1_COMPUTE_GEOMOPT:
		DoGeomOpt();
		break;
		
		case SIGNAL_MM1_COMPUTE_MOLDYN:
		DoMolDyn();
		break;
		
		case SIGNAL_MM1_OBJECTS_RIBBON:
		mm1_mdl::amino_builder.Identify(this);
		DefineSecondaryStructure(this);
		
		AddObject(new mm1_ribbon(this, win->colormode, 0, 4));		// min. order is 2!!!
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_OBJECTS_EPLANE:
		cout << "give dimension of the plane [nm] (1.0 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) mm1_GetESPValue; cpp.cf = GetRGBRange2;
		cpp.value = 4.1868 * 33.20716; cpp.descr = "ESP-";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_OBJECTS_ESURFACE:	tmp1 = 4.1868 * 33.20716;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) mm1_GetESPValue; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetRedColor; csp.svalue = +tmp1; csp.descr = "pos ESP-";
		AddObject(new color_surface(ol_static(), csp));
		
		csp.cf = GetBlueColor; csp.svalue = -tmp1; csp.descr = "neg ESP-";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_MM1_OBJECTS_VDWSURFACE:	tmp1 = 4.1868 * 33.20716 / 2.0;
		cout << "give dimension of the surface [nm] (2.5 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false; csp.cf = GetRGBRange2;
		csp.vf1 = (ValueFunction *) mm1_GetSASValue; csp.vf2 = (ValueFunction *) mm1_GetESPValue;
		csp.svalue = 1.0; csp.cvalue = tmp1; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		csp.descr = "ESP-colored VDW-";
		
		AddObject(new color_surface(ol_static(), csp));
		UpdateAllViewWindows();
		break;
		
		default:
		cout << "could not handle signal 0x" << hex << signal << dec << endl;
	}
}

void t3_mm1_gui::EnergyReport(mm1_eng * eng)
{
	cout << "energy = " << eng->energy << " kJ/mol" << endl;
}

bool t3_mm1_gui::GeomOptGetParam(mm1_geomopt_param * param)
{
	cout << "TODO: a GTK+ dialog box!!!!!" << endl;
	cout << "how many cg-steps (500 might be Ok) ??? ";
	cin >> param->nsteps;
	
	return true;
}

#define START 10000	// see mm1gui.cpp!!!!!!!!

bool t3_mm1_gui::MolDynGetParam(mm1_moldyn_param * param)
{
	cout << "TODO: a GTK+ dialog box!!!!!" << endl;
	cout << "how many md-steps for simulation (10000 might be Ok) ??? ";
	cin >> param->nsteps;
	
	cout << "there are also " << START << " heating steps, total steps = " << (START + param->nsteps) << endl;
	cout << "the trajectory file will contain " << (param->nsteps / 100) << " frames!!!" << endl;
	
	cout << "time-step length [fs] (0.5 or less might be Ok) ??? ";
	cin >> param->timestep;
	
	cout << "traj-file name ??? ";
	cin >> param->filename;
	
	return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
