// T3QM2GUI.CPP

// Copyright (C) 2000 Tommi Hassinen.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "t3qm2gui.h"

#include "t3dialog.h"
#include "t3window.h"

#include "qm2eng1.h"
#include "qm2eng9.h"

#include "plane.h"
#include "surface.h"

#include "color.h"
#include "window.h"

////////////////////////////////////////////////////////////////////////////////////////////////////

GnomeUIInfo t3_qm2_gui::fileimport_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "mm1...", "mm1",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_IMPORT_MM1, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::file_submenu[] =
{
	GNOMEUIINFO_MENU_OPEN_ITEM(gtkgl_window::GNOMEPopupHandler, SIGNAL_QM2_FILE_OPEN),
	GNOMEUIINFO_SUBTREE("_Import", fileimport_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_SAVE_AS_ITEM(gtkgl_window::GNOMEPopupHandler, SIGNAL_QM2_FILE_SAVEAS),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_MENU_CLOSE_ITEM(gtkgl_window::GNOMEPopupHandler, SIGNAL_QM2_FILE_CLOSE),
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::select_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Select All", "select all atoms...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_SELECT_SELECTALL, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Invert Selection", "invert the selection...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_SELECT_INVERTSELECTION, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::settings_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Set Current Element", "el...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_SETTINGS_ELEMENT, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::viewscreate_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "New Camera", "create a new view with a new camera...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_VIEWS_NEWCAMERA, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "New View", "create a new view using the current camera...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_VIEWS_NEWVIEW, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::views_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_Create", viewscreate_submenu),
	
	{
		GNOME_APP_UI_ITEM, "Delete This View", "remove this view...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_VIEWS_DELETEVIEW, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::lights_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Create New Light", "create a new light object...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_LIGHTS_NEWLIGHT, NULL
	},
	
	GNOMEUIINFO_SEPARATOR,
	
	{
		GNOME_APP_UI_ITEM, "Switch Local Lights", "loc",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_LIGHTS_SWITCHLOCAL, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Switch Global Lights", "glob",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_LIGHTS_SWITCHGLOBAL, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::rendermode_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Wireframe", "wf",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_RMODE_WIREFRAME, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Ball & Stick", "bs",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_RMODE_BALLANDSTICK, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Nothing", "null",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_RMODE_NOTHING, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::projection_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Orthographic", "ortho",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_PROJECTION_ORTHOGRAPHIC, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Perspective", "persp",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_PROJECTION_PERSPECTIVE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::render_submenu[] =
{
	GNOMEUIINFO_SUBTREE("_Views", views_submenu),
	GNOMEUIINFO_SUBTREE("_Lights", lights_submenu),
	GNOMEUIINFO_SUBTREE("_Rendering Mode", rendermode_submenu),
	GNOMEUIINFO_SUBTREE("_Projection", projection_submenu),
	
	{
		GNOME_APP_UI_ITEM, "Quick Update Mode", "quick update...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_RENDER_QUICKUPDATE, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::objects_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Electron density plane", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_OBJECTS_EDPLANE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Electron density surface", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_OBJECTS_EDSURFACE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Molecular orbital plane", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_OBJECTS_MOPLANE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "Molecular orbital surface", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_OBJECTS_MOSURFACE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "MO-density plane", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_OBJECTS_MODPLANE, NULL
	},
	
	{
		GNOME_APP_UI_ITEM, "MO-density surface", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_OBJECTS_MODSURFACE, NULL
	},
	
	GNOMEUIINFO_SEPARATOR,
	
	{
		GNOME_APP_UI_ITEM, "Delete Current Object", "delete",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_GEN_OBJECTS_DELCURROBJ, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::compute_submenu[] =
{
	{
		GNOME_APP_UI_ITEM, "Energy", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_COMPUTE_ENERGY, NULL
	},
	
	GNOMEUIINFO_END
};

GnomeUIInfo t3_qm2_gui::popupmenu_info[] =
{
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	// a "tearoff" item would be nice here at the beginning !?!?!?!??!?!?!?!?!?!?!?!!!!???!??!!!
	
	GNOMEUIINFO_SUBTREE("_File", file_submenu),
	GNOMEUIINFO_SUBTREE("_Select", select_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Settings...", settings_submenu),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_SUBTREE("_Render", render_submenu),
	GNOMEUIINFO_SUBTREE("_Objects", objects_submenu),
	GNOMEUIINFO_SUBTREE("_Compute", compute_submenu),
	
	{
		GNOME_APP_UI_ITEM, "Set Current Orbital", "...",
		gtkgl_window::GNOMEPopupHandler, (void *) SIGNAL_QM2_SETORBITAL, NULL
	},
	
	GNOMEUIINFO_END
};

////////////////////////////////////////////////////////////////////////////////////////////////////

t3_qm2_gui::t3_qm2_gui(window_factory & p1, ostream * p2) : qm2_gui(p1, p2), bs_mdl_simple(p2)
{
	popupmenu = gnome_popup_menu_new(popupmenu_info);
}

t3_qm2_gui::~t3_qm2_gui(void)
{
}

void t3_qm2_gui::PopupEvent(view_window * win, int signal)
{
	fGL tmp1; fGL tmp2;
//	i32s sz[3] = { 50, 50, 50 };
i32s sz[3] = { 25, 25, 25 };					// make them quickly...
	fGL dm[3] = { 0.55, 0.50, 0.45 };
	
	cp_param cpp;
//	cpp.np = 30; cpp.gui = this; cpp.ref = current_eng;
cpp.np = 15; cpp.gui = this; cpp.ref = current_eng;		// make them quickly...

	cs_param csp;
	csp.np = sz; csp.dim = dm; csp.ostr = & cout;
	csp.gui = this; csp.ref = current_eng;
	
	char input;
	
	switch (signal)
	{
		case SIGNAL_QM2_IMPORT_MM1:
		cout << "name of MM1 file to import ??? ";
		cin >> buffer; ImportMM1(buffer);
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_QM2_SETTINGS_ELEMENT:
		new element_dialog();			// memory leak???
		break;
		
		case SIGNAL_QM2_SETORBITAL:
		cout << "current orbital is now " << current_orbital << endl;
		cout << "enter new value (BE CAREFUL, THERE'S NO BOUNDS CHECKING YET!!!) : ";
		
		cin >> current_orbital;
		
		cout << "current orbital now set to " << current_orbital << endl;
		break;
		
		case SIGNAL_QM2_RMODE_WIREFRAME:
		win->render = RENDER_WIREFRAME; UpdateAllViewWindows();
		break;
		
		case SIGNAL_QM2_RMODE_BALLANDSTICK:
		win->render = RENDER_BALL_AND_STICK; UpdateAllViewWindows();
		break;
		
		case SIGNAL_QM2_RMODE_NOTHING:
		win->render = RENDER_NOTHING; UpdateAllViewWindows();
		break;
		
// from now on we are 100% dependent on MPQC...
// from now on we are 100% dependent on MPQC...
// from now on we are 100% dependent on MPQC...

#ifdef ENABLE_MPQC

		case SIGNAL_QM2_COMPUTE_ENERGY:
		DoEnergy();
		break;
		
// this "cpp.ref"-thing is not yet finished. won't work with other engines than mpqc...
// this "cpp.ref"-thing is not yet finished. won't work with other engines than mpqc...
// this "cpp.ref"-thing is not yet finished. won't work with other engines than mpqc...

		case SIGNAL_QM2_OBJECTS_EDPLANE:
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) qm2_eng_mpqc::getElDens; cpp.cf = GetGreenColor;
		cpp.value = 0.05; cpp.descr = "E-density ";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_QM2_OBJECTS_EDSURFACE: tmp1 = 0.05;
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) qm2_eng_mpqc::getElDens; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetGreenColor; csp.svalue = +tmp1; csp.descr = "E-density ";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_QM2_OBJECTS_MOPLANE:
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) qm2_eng_mpqc::getOrbital; cpp.cf = GetRGBRange2;
		cpp.value = 0.05; cpp.descr = "molecular orbital ";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_QM2_OBJECTS_MOSURFACE: tmp1 = 0.05;
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) qm2_eng_mpqc::getOrbital; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetRedColor; csp.svalue = +tmp1; csp.descr = "pos MO ";
		AddObject(new color_surface(ol_static(), csp));
		
		csp.cf = GetBlueColor; csp.svalue = -tmp1; csp.descr = "neg MO ";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_QM2_OBJECTS_MODPLANE:
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the plane [nm] (0.5 might be ok) ??? "; cin >> tmp1;
		
		cpp.dim = tmp1; cpp.transparent = true;
		cpp.vf = (ValueFunction *) qm2_eng_mpqc::getOrbDens; cpp.cf = GetGreenColor;
		cpp.value = 0.0125; cpp.descr = "MO-density ";
		
		AddObject(new color_plane(ol_static(), cpp));
		UpdateAllViewWindows();
		break;
		
		case SIGNAL_QM2_OBJECTS_MODSURFACE: tmp1 = 0.0125;
if (!current_eng) { cout << "calculate energy first!!!" << endl; break; }
		cout << "give dimension of the surface [nm] (1.0 might be ok) ??? ";
		cin >> tmp2; dm[0] = dm[1] = dm[2] = tmp2;
		
		cout << "choose the type of surface : solid or wireframe ? (s/w) ";
		cin >> input; csp.wireframe = (input == 'w' || input == 'W');
		
		csp.transparent = false;
		csp.vf1 = (ValueFunction *) qm2_eng_mpqc::getOrbDens; csp.vf2 = GetUnity;
		csp.cvalue = 1.0; csp.toler = 1.0e-6 * tmp1; csp.maxc = 1000;
		
		csp.cf = GetGreenColor; csp.svalue = +tmp1; csp.descr = "MO-density ";
		AddObject(new color_surface(ol_static(), csp));
		
		UpdateAllViewWindows();
		break;
		
#endif	// ENABLE_MPQC

	}
}

void t3_qm2_gui::EnergyReport(qm2_eng * eng)
{
	cout << "energy = " << eng->energy << " Hartrees" << endl;
}

////////////////////////////////////////////////////////////////////////////////////////////////////

// eof
