// T3WINDOW.H : a view window class for GTK+/GNOME.

// Copyright (C) 1999 Tommi Hassinen, Ville Heikkil.

// This program is free software; you can redistribute it and/or modify it
// under the terms of the license (GNU GPL) which comes with this package.

////////////////////////////////////////////////////////////////////////////////////////////////////

#include "config.h"	// this is for TARGET3...

#ifndef T3WINDOW_H
#define T3WINDOW_H

class gtkgl_window;
class gtkgl_view_window;

class gtkgl_factory;

////////////////////////////////////////////////////////////////////////////////////////////////////

#include <gtk/gtk.h>
#include <gtkgl/gtkglarea.h>

#include "window.h"

#include <vector>

////////////////////////////////////////////////////////////////////////////////////////////////////

#define SIGNAL_GEN_SELECT_SELECTALL			0x0f005012
#define SIGNAL_GEN_SELECT_INVERTSELECTION		0x0f005013

#define SIGNAL_GEN_PROJECTION_ORTHOGRAPHIC		0x0f005028
#define SIGNAL_GEN_PROJECTION_PERSPECTIVE		0x0f005029
#define SIGNAL_GEN_RENDER_QUICKUPDATE			0x0f005030

#define SIGNAL_GEN_VIEWS_NEWCAMERA			0x0f005020
#define SIGNAL_GEN_VIEWS_NEWVIEW			0x0f005021
#define SIGNAL_GEN_VIEWS_DELETEVIEW			0x0f005022

#define SIGNAL_GEN_LIGHTS_NEWLIGHT			0x0f005025
#define SIGNAL_GEN_LIGHTS_SWITCHLOCAL 			0x0f005026
#define SIGNAL_GEN_LIGHTS_SWITCHGLOBAL			0x0f005027

#define SIGNAL_GEN_OBJECTS_DELCURROBJ			0x0f005052

////////////////////////////////////////////////////////////////////////////////////////////////////

class gtkgl_window : virtual public window
{
	protected:
	
	GtkWidget * gtkwin;
	GtkWidget * glarea;	// this is the main widget!!!
	
	private:
	
	static vector<gtkgl_window *> gtkgl_vector;
	
	public:
	
	gtkgl_window(bs_gui *);
	virtual ~gtkgl_window(void);
	
	void CreateWindow(void);		// virtual
	void SetTitle(const char *);		// virtual
	void SetCurrent(void);			// virtual
	void Update(bool);			// virtual
	
	private:
	
	virtual void ExposeEvent(void) = 0;
	
	private:
	
	static gtkgl_window * GetWindow(GtkWidget *);
	
	static gint ExposeHandler(GtkWidget *, GdkEventExpose *);
	static void PopupHandler(GtkWidget *, i32s);
	
	static gint ButtonHandler(GtkWidget *, GdkEventButton *);
	static gint MotionNotifyHandler(GtkWidget *, GdkEventMotion *);
	static gint ConfigureHandler(GtkWidget *, GdkEventConfigure *);
	static void DestroyHandler(GtkObject *);
	
	public:
	
	// we can't directly use the above private popup handler in GNOME widgets,
	// so we must define a public one here. might be useful in debugging etc...
	
	static void GNOMEPopupHandler(GtkWidget *, void *);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class gtkgl_view_window : public gtkgl_window, public view_window
{
	public:
	
	gtkgl_view_window(bs_gui *, camera *);
	~gtkgl_view_window(void);
	
	private:
	
	void ExposeEvent(void);		// virtual
};

////////////////////////////////////////////////////////////////////////////////////////////////////

class gtkgl_factory : public window_factory
{
	private:
	
	static gtkgl_factory * instance;
	gtkgl_factory(void);
	
	public:
	
	~gtkgl_factory(void);
	static gtkgl_factory * GetInstance(void);
	
	view_window * ProduceView(bs_gui *, camera *);
};

////////////////////////////////////////////////////////////////////////////////////////////////////

#endif  // T3WINDOW_H

// eof
